# ncert_learn/__init__.py
__version__ = "4.5.10"

from .intfncs import checkprime,factors,len_fibonacci,checkarmstrong,intreverse,checkpalindrome,checkstrong,checkniven,prime,armstrong,strong,niven,palindrome,len_armstrong,len_strong,len_niven,len_palindrome,checkeven,checkodd,checkzero,checknegative,checkpositive,len_prime
from .stkdict import clearstackdict,createstackdict,pushstackdict,popstackdict,peekstackdict,displaymodestackdict# Import functions
from .stklist import createstacklst,clearstacklst,pushstacklst,popstacklst,peekstacklst,displaymodestacklst
from .dat import studyexamples,studyfunctions,studyshort,studymysqlpython,notes,basics,mysqlnotes
from .area import areaofcircle,areaofpolygon,areaofrectangle,areaofsquare,areaoftriangle
from .conversions import integertobinary,integertooctal,integertohexadecimal,binarytointeger
from .mysql_ import mysql_execute_advanced_mode,mysqlconnect,mysqlconnectwithdatabase,mysqlshowdatabases,mysqlcreatedatabase,mysqlshowtables,mysqlshowtablesfromdatabase,mysqldescribetable,mysqldescribetablefromdatabase,mysqlfetchalltable,mysqlfetchalltablefromdatabase,mysqlcreatetable,mysqltableinsert,mysqlrowcounttablefromdatabase,mysqlexecutequery
from .sort import bubblesort,insertionsort
from .text import istextfile,copytextfromonetoanother,opentextfile,addlinetofile,readtextfile,cleartextfile,readspecificline,overwritetextfile,text_file_operations_advanced_mode
from .os import getexecutablepath,cpucount,listdir,listdirfrompath,osname,processorname,isnetworkconnected,getpythoninterpreter
from .packageinstall import install_library,install_librariesfromlist
from .support import supportemail,feedbackemail,bugtrackerurl,githuburl,documentationurl,authorname
from .time import currenttime,processtime,monotonic,threadtime
from .binary import isbinaryfile,copybinaryfromonetoanother,openbinaryfile,addtextobinaryfile,readbinaryfile,clearbinaryfile,readbinaryspecificchunk,overwritebinaryfile,binary_file_operations_advanced_mode
from .csv import iscsvfile,copycsvfromonetoanother,opencsvfile,addrowtocsv,readcsvfile,clearcsvfile,readcsvspecificline,overwritecsvfile,csv_file_operations_advanced_mode
from .xampp import check_xampp_files,is_process_running_xampp,stop_xampp_mysql,stop_xampp_apache,start_xampp_mysql,start_xampp_apache,start_xampp_mysql,start_xampp_apache,check_phpmyadmin_accessible,start_xampp_and_open_phpmyadmin
from .nmapinstall import  install_and_setup_nmap
from .zip import zip_is_supported_file,zip_is_supported,zip_list_files,zip_extract_file,zip_add_file,zip_extract_all
# __init__.py in your sqlmap directory
from .fileaccess import check_file_existence,create_directory,download_file,get_file_size,get_file_last_modified,rename_file,delete_file,move_file,extract_zip,compress_files,get_url_status,fetch_url_content,download_files_from_urls,get_files_in_directory,count_lines_in_file,get_current_datetime,get_extension,get_file_name_without_extension,get_file_type,move_files_to_directory
from .sqlmap import (
    sqlmap_scan, sqlmap_api_call, sqlmap_scan_for_union_based_sql_injection, 
    sqlmap_scan_for_error_based_sql_injection, sqlmap_scan_for_time_based_sql_injection,
    sqlmap_scan_for_boolean_blind_sql_injection, sqlmap_scan_for_stacked_queries, 
    sqlmap_scan_for_tautology_based_sql_injection, sqlmap_scan_for_second_order_sql_injection,
    sqlmap_scan_for_inline_query_sql_injection, sqlmap_scan_for_blind_time_based_sql_injection,
    sqlmap_scan_with_cookie_based_auth, sqlmap_scan_with_jwt_token, sqlmap_scan_with_basic_auth,
    sqlmap_scan_with_digest_auth, sqlmap_scan_with_ntlm_auth, sqlmap_scan_with_custom_proxy,
    sqlmap_scan_with_socks_proxy, sqlmap_scan_with_multiple_proxies, sqlmap_scan_with_custom_user_agent,
    sqlmap_scan_with_timeout, sqlmap_scan_with_max_redirects, sqlmap_scan_with_retries,
    sqlmap_scan_with_quiet_mode, sqlmap_scan_with_verbosity, sqlmap_scan_for_os_command_injection,
    sqlmap_scan_for_os_command_injection_using_shell, sqlmap_scan_for_os_command_injection_with_wildcards,
    sqlmap_scan_for_local_file_inclusion, sqlmap_scan_for_remote_file_inclusion, sqlmap_scan_for_file_read,
    sqlmap_scan_for_file_upload, sqlmap_scan_for_reflected_xss, sqlmap_scan_for_stored_xss, 
    sqlmap_scan_for_dom_xss, sqlmap_scan_for_mysql_version, sqlmap_scan_for_postgresql_version, 
    sqlmap_scan_for_oracle_version, sqlmap_scan_for_mssql_version, sqlmap_scan_with_error_handling,
    sqlmap_scan_with_logging, sqlmap_api_status_check, sqlmap_scan_with_custom_payloads,
    sqlmap_scan_with_network_error_simulation, sqlmap_scan_with_json_output, bulk_sqlmap_scan, 
    sqlmap_api_start_scan, sqlmap_api_get_scan_status, sqlmap_scan_with_timeout_and_retries, 
    sqlmap_api_terminate_scan, sqlmap_scan_for_union_based_sql_injection, sqlmap_scan_for_syntax_based_sql_injection,
    sqlmap_scan_for_time_based_sql_injection, sqlmap_scan_for_error_based_sql_injection, sqlmap_scan_for_blind_sql_injection,
    sqlmap_scan_with_response_timeout, sqlmap_scan_with_retry, sqlmap_scan_with_max_redirects, 
    sqlmap_scan_with_basic_auth, sqlmap_scan_with_digest_auth, sqlmap_scan_with_ntlm_auth, 
    sqlmap_scan_with_custom_payload, sqlmap_scan_with_multiple_payloads, sqlmap_scan_with_custom_proxy, 
    sqlmap_scan_with_socks_proxy, sqlmap_scan_with_multiple_proxies, sqlmap_scan_with_quiet_mode,
    sqlmap_scan_with_verbosity, sqlmap_scan_with_output_log, sqlmap_scan_with_sqli_check,
    sqlmap_scan_with_xss_check, sqlmap_scan_for_file_inclusion, sqlmap_scan_for_remote_file_inclusion,
    sqlmap_scan_with_stacked_queries, sqlmap_scan_with_sqli_privesc, sqlmap_scan_for_boolean_blind, 
    sqlmap_scan_for_time_based_blind, sqlmap_scan_with_union_query_injection, sqlmap_scan_for_directory_traversal, 
    sqlmap_scan_for_file_read, sqlmap_scan_for_file_upload, sqlmap_scan_with_custom_headers,
    sqlmap_scan_with_custom_cookies, sqlmap_scan_with_user_agent, sqlmap_scan_for_mysql_version, 
    sqlmap_scan_for_postgresql_version, sqlmap_scan_for_oracle_version, sqlmap_scan_for_mssql_version, 
    sqlmap_scan_for_reflected_xss, sqlmap_scan_for_stored_xss, sqlmap_scan_for_dom_xss, 
    sqlmap_scan_with_session_storage, sqlmap_scan_with_cookie_based_auth, sqlmap_scan_with_basic_auth,
    sqlmap_scan_with_jwt_auth, sqlmap_cleanup, sqlmap_scan_for_union_based_sql_injection,
    sqlmap_scan_for_error_based_sql_injection, sqlmap_scan_for_time_based_sql_injection, 
    sqlmap_scan_for_boolean_blind_sql_injection, sqlmap_scan_for_stack_based_sql_injection, 
    sqlmap_scan_for_tautology_based_sql_injection, sqlmap_scan_with_cookie_based_auth, sqlmap_scan_with_jwt_token,
    sqlmap_scan_with_basic_auth, sqlmap_scan_with_digest_auth, sqlmap_scan_with_ntlm_auth, sqlmap_scan_with_custom_proxy,
    sqlmap_scan_with_socks_proxy, sqlmap_scan_with_multiple_proxies, sqlmap_scan_with_custom_user_agent, 
    sqlmap_scan_with_timeout, sqlmap_scan_with_max_redirects, sqlmap_scan_with_retries, 
    sqlmap_scan_with_quiet_mode, sqlmap_scan_with_verbosity, sqlmap_scan_for_boolean_based_blind_injection,
    sqlmap_scan_for_time_based_blind_injection, sqlmap_scan_for_stack_based_blind_injection, 
    sqlmap_scan_for_os_command_injection, sqlmap_scan_for_os_command_injection_using_shell,
    sqlmap_scan_for_os_command_injection_with_wildcards, sqlmap_scan_for_local_file_inclusion, 
    sqlmap_scan_for_remote_file_inclusion, sqlmap_scan_for_file_read, sqlmap_scan_for_file_upload, 
    sqlmap_cleanup, sqlmap_scan_with_custom_payload, sqlmap_scan_with_multiple_payloads, 
    sqlmap_scan_with_random_payloads, sqlmap_scan_for_reflected_xss, sqlmap_scan_for_stored_xss, 
    sqlmap_scan_for_dom_xss, sqlmap_scan_for_mysql_version, sqlmap_scan_for_postgresql_version,
    sqlmap_scan_for_oracle_version, sqlmap_scan_for_mssql_version, sqlmap_scan_with_error_handling,
    sqlmap_scan_for_union_based_sql_injection, sqlmap_scan_for_error_based_sql_injection,
    sqlmap_scan_for_time_based_sql_injection, sqlmap_scan_for_boolean_blind_sql_injection,
    sqlmap_scan_for_stacked_queries, sqlmap_scan_for_second_order_sql_injection, 
    sqlmap_scan_with_cookie_based_auth, sqlmap_scan_with_jwt_token, sqlmap_scan_with_basic_auth,
    sqlmap_scan_with_digest_auth, sqlmap_scan_with_ntlm_auth, sqlmap_scan_with_custom_proxy,
    sqlmap_scan_with_socks_proxy, sqlmap_scan_with_multiple_proxies, sqlmap_scan_with_custom_user_agent,
    sqlmap_scan_with_timeout, sqlmap_scan_with_max_redirects, sqlmap_scan_with_retries, 
    sqlmap_scan_with_quiet_mode, sqlmap_scan_with_verbosity, sqlmap_scan_for_os_command_injection,
    sqlmap_scan_for_os_command_injection_using_shell, sqlmap_scan_for_os_command_injection_with_wildcards,
    sqlmap_scan_for_local_file_inclusion, sqlmap_scan_for_remote_file_inclusion, sqlmap_scan_for_file_read,
    sqlmap_scan_for_file_upload, sqlmap_cleanup, sqlmap_scan_with_custom_payload, 
    sqlmap_scan_with_multiple_payloads, sqlmap_scan_with_random_payloads, sqlmap_scan_for_reflected_xss,
    sqlmap_scan_for_stored_xss, sqlmap_scan_for_dom_xss, sqlmap_scan_for_mysql_version, 
    sqlmap_scan_for_postgresql_version, sqlmap_scan_for_oracle_version, sqlmap_scan_for_mssql_version,
    sqlmap_scan_with_error_handling
)
from .googlesearch_ import getgoogleanswer,get_google_answer
from .nmap import (nmap_scan,nmap_intense_scan,nmap_quick_scan,nmap_ping_sweep,nmap_os_detection,nmap_service_version_detection,nmap_tcp_connect_scan,nmap_syn_scan,nmap_udp_scan,nmap_traceroute_scan,nmap_vulnerability_scan,nmap_stealth_scan,nmap_ack_scan,nmap_window_scan,nmap_masquerade_scan,nmap_decoy_scan,nmap_fin_scan,nmap_xmas_scan,nmap_null_scan,nmap_ping_scan,nmap_sweep_scan,nmap_ftp_scan,nmap_smtp_scan,nmap_dns_scan,nmap_http_scan,nmap_pop3_scan,nmap_ssh_scan,nmap_mysql_scan,nmap_smb_vuln_scan,nmap_http_favicon_scan,nmap_smb_share_scan,nmap_traffic_analysis,nmap_smtp_vuln_scan,nmap_rdp_vuln_scan,nmap_irc_scan,nmap_ldap_scan,nmap_snmp_scan,nmap_dns_vuln_scan,nmap_http_auth_scan,nmap_telnet_scan,nmap_mysql_brute_scan,nmap_smb_info_scan,nmap_nfs_scan,nmap_mongodb_scan,nmap_dhcp_scan,nmap_vnc_scan,nmap_ftp_anonymous_scan,nmap_smtp_enum_scan,nmap_ftp_banners_scan,nmap_smb_enum_users_scan,nmap_ssh_enum_users_scan,nmap_pop3_brute_scan,nmap_tcp_ping_scan,nmap_ssl_scan,nmap_ssl_vuln_scan,nmap_smb_file_scan,nmap_ftp_banners_scan,nmap_vuln_detection_scan,nmap_smtp_open_scan,nmap_http_methods_scan,nmap_ssh_vuln_scan,nmap_http_xss_scan,nmap_ftp_anonymous_scan,nmap_dhcp_discovery_scan,nmap_nfs_mount_scan,nmap_smtp_brute_scan,nmap_smb_brute_scan,nmap_ldap_enum_scan,nmap_http_sitemap_scan,nmap_ftp_version_scan,nmap_snmp_community_scan,nmap_ssh_version_scan,nmap_mysql_info_scan,nmap_dns_zone_transfer_scan,nmap_pop3_vuln_scan,nmap_telnet_vuln_scan,nmap_imap_brute_scan,nmap_irc_banners_scan,nmap_http_ssl_scan,nmap_smb_vuln2_scan,nmap_http_robots_scan,nmap_mssql_brute_scan,nmap_smb_security_scan,nmap_smtp_starttls_scan,nmap_ftp_bypass_scan,nmap_dhcp_server_scan,nmap_dns_traceroute_scan,nmap_smtp_enum_vuln_scan,nmap_dns_dos_scan,nmap_rpc_scan,nmap_mongodb_bypass_scan,nmap_smb_enum_shares_scan,nmap_fingerprint_scan,nmap_snmp_enum_users_scan,nmap_ssl_certificate_scan,nmap_dhcp_lease_scan,nmap_mysql_enum_scan,nmap_http_security_scan,nmap_telnet_enum_scan,nmap_ftp_enumerate_scan,nmap_smb_enum_users_scan,nmap_pop3_enum_scan,nmap_smtp_enum_scan,nmap_http_cookie_scan,nmap_dhcp_fingerprint_scan,nmap_smb_enum_shares_users_scan,nmap_ftp_version_vuln_scan,nmap_smb_session_scan,nmap_telnet_enumerate_scan,nmap_smtp_vuln_scan,nmap_finger_scan,nmap_ftp_login_scan,nmap_ssh_brute_scan,nmap_http_ssl_cc_scan,nmap_smtp_starttls_vuln_scan,nmap_https_ping_scan,nmap_http_user_agent_scan,nmap_http_open_redirect_scan,nmap_smb_domain_info_scan,nmap_rdp_scan,nmap_dns_recursion_scan,nmap_http_ssl_scan,nmap_mongodb_auth_scan,nmap_http_basic_auth_brute_scan,nmap_ftp_vuln_scan,nmap_snmp_enumerate_users_scan,nmap_ssl_scan,nmap_netbios_scan,nmap_mysql_enum_vuln_scan,nmap_dns_scan,nmap_fingerprint_scan,nmap_smb_enum_machine_scan,nmap_smtp_tls_vuln_scan)
from .cmd import(execute_command,cmd_list_files,cmd_get_disk_usage,cmd_get_memory_usage,cmd_get_running_processes,cmd_kill_process,cmd_create_directory,cmd_delete_file_or_directory,cmd_get_system_uptime,cmd_get_kernel_version,cmd_get_system_load,cmd_check_for_updates,cmd_install_package,cmd_uninstall_package,cmd_get_active_users,cmd_get_ip_address,cmd_check_firewall_status,cmd_flush_dns_cache,cmd_get_system_temperature,cmd_reset_tcpip_stack,cmd_check_disk_for_errors,cmd_restart_system,cmd_shutdown_system,cmd_open_control_panel,cmd_open_task_manager,cmd_check_services,cmd_start_service,cmd_stop_service,cmd_view_env_variables)
from .getfunctions import get_function_names_from_python_file
from .engine import engine_save_model,engine_load_model,engine_tensor_add,engine_tensor_dot,engine_sigmoid_activation,engine_dense_layer,engine_sgd_optimizer,engine_mse_loss,test_engine_functions
from .pdfextract import extract_text_from_pdf,save_text_to_file
from .lstfncs import list_append_item, list_remove_item, list_insert_item, list_pop_item, list_find_index, list_contains_item, list_sort, list_reverse, list_clear, list_copy, list_extend, list_count, list_min, list_max, list_sum, list_mean, list_unique, list_combine, list_difference, list_intersection, list_is_empty
from .strfncs import str_reverse, str_to_upper, str_to_lower, str_is_palindrome, str_count_occurrences, str_is_alpha, str_is_digit, str_find_substring, str_replace_substring, str_split_words, str_strip_spaces, str_startswith, str_endswith, str_isalnum, str_title_case, str_concat, str_join
from .dictfncs import dict_add_key_value, dict_remove_key, dict_get_value, dict_update_value, dict_contains_key, dict_get_all_keys, dict_get_all_values, dict_clear, dict_copy, dict_items, dict_pop_item, dict_update, dict_setdefault, dict_fromkeys, dict_get_key_with_max_value, dict_get_key_with_min_value
from .advancedmodeforpg import adv_gcd, adv_lcm, adv_prime_factors, adv_is_prime, adv_modular_exponentiation, adv_is_perfect_square, adv_fast_fourier_transform, adv_hash_string, adv_fast_modular_inverse, adv_fibonacci, adv_sieve_of_eratosthenes, adv_modular_square_root, adv_random_prime, adv_sum_of_squares, adv_calculate_modular_power, adv_combinations, adv_permutations
# Optional: List functions to include in 'import *' cal
__all__ = ['mysqlconnectwithdatabase','exampleusage','checkprime','factors','len_fibonacci','checkarmstrong','intreverse','checkpalindrome','checkstrong','checkniven','prime','armstrong','strong','niven','palindrome','len_armstrong','len_strong','len_niven','len_palindrome','checkeven','checkodd','checkzero','checknegative','checkpositive','clearstackdict','createstackdict','pushstackdict','popstackdict','peekstackdict','displaymodestackdict','createstacklst','clearstacklst','pushstacklst','popstacklst','peekstacklst','displaymodestacklst','studyexamples','studyfunctions','studyshort','studymysqlpython','notes','basics','extendednotes','mysqlnotes'
           ,'bubblesort','insertionsort','mysqlconnect','mysqlshowdatabases','mysqlcreatedatabase','mysqlshowtables','mysqlshowtablesfromdatabase','mysqldescribetable','mysqldescribetablefromdatabase','mysqlfetchalltable','mysqlfetchalltablefromdatabase','mysqlcreatetable','mysqltableinsert','mysqlrowcounttablefromdatabase','mysqlexecutequery',
           'integertobinary','integertooctal','integertohexadecimal','binarytointeger','areaofcircle','areaofpolygon','areaofrectangle','areaofsquare','areaoftriangle',
           'istextfile','copytextfromonetoanother','opentextfile','addlinetofile','readtextfile','cleartextfile','readspecificline',
           'getexecutablepath','cpucount','listdir','listdirfrompath','osname','processorname','isnetworkconnected','getpythoninterpreter',
           'install_library','install_librariesfromlist',
           'supportemail','feedbackemail','bugtrackerurl','githuburl','documentationurl','authorname',
           'currenttime','processtime','monotonic','threadtime',
           'isbinaryfile','copybinaryfromonetoanother','openbinaryfile','addtextobinaryfile','readbinaryfile','clearbinaryfile','readbinaryspecificchunk',
           'iscsvfile','copycsvfromonetoanother','opencsvfile','addrowtocsv','readcsvfile','clearcsvfile','readcsvspecificline',
           'check_xampp_files','is_process_running_xampp','stop_xampp_mysql','stop_xampp_apache','start_xampp_mysql','start_xampp_apache','start_xampp_mysql','start_xampp_apache','check_phpmyadmin_accessible','start_xampp_and_open_phpmyadmin',
           'zip_is_supported_file','zip_is_supported','zip_list_files','zip_extract_file','zip_add_file','zip_extract_all',
           'sqlmap_scan', 'sqlmap_api_call', 'sqlmap_scan_for_union_based_sql_injection', 'sqlmap_scan_for_error_based_sql_injection', 'sqlmap_scan_for_time_based_sql_injection', 'sqlmap_scan_for_boolean_blind_sql_injection', 'sqlmap_scan_for_stacked_queries', 'sqlmap_scan_for_tautology_based_sql_injection', 'sqlmap_scan_for_second_order_sql_injection', 'sqlmap_scan_for_inline_query_sql_injection', 'sqlmap_scan_for_blind_time_based_sql_injection', 'sqlmap_scan_with_cookie_based_auth', 'sqlmap_scan_with_jwt_token', 'sqlmap_scan_with_basic_auth', 'sqlmap_scan_with_digest_auth', 'sqlmap_scan_with_ntlm_auth', 'sqlmap_scan_with_custom_proxy', 'sqlmap_scan_with_socks_proxy', 'sqlmap_scan_with_multiple_proxies', 'sqlmap_scan_with_custom_user_agent', 'sqlmap_scan_with_timeout', 'sqlmap_scan_with_max_redirects', 'sqlmap_scan_with_retries', 'sqlmap_scan_with_quiet_mode', 'sqlmap_scan_with_verbosity', 'sqlmap_scan_for_os_command_injection', 'sqlmap_scan_for_os_command_injection_using_shell', 'sqlmap_scan_for_os_command_injection_with_wildcards', 'sqlmap_scan_for_local_file_inclusion', 'sqlmap_scan_for_remote_file_inclusion', 'sqlmap_scan_for_file_read', 'sqlmap_scan_for_file_upload', 'sqlmap_scan_for_reflected_xss', 'sqlmap_scan_for_stored_xss', 'sqlmap_scan_for_dom_xss', 'sqlmap_scan_for_mysql_version', 'sqlmap_scan_for_postgresql_version', 'sqlmap_scan_for_oracle_version', 'sqlmap_scan_for_mssql_version', 'sqlmap_scan_with_error_handling', 'sqlmap_scan_with_logging', 'sqlmap_api_status_check', 'sqlmap_scan_with_custom_payloads', 'sqlmap_scan_with_network_error_simulation', 'sqlmap_scan_with_json_output', 'bulk_sqlmap_scan', 'sqlmap_api_start_scan', 'sqlmap_api_get_scan_status', 'sqlmap_scan_with_timeout_and_retries', 'sqlmap_api_terminate_scan', 'sqlmap_scan_for_union_based_sql_injection', 'sqlmap_scan_for_syntax_based_sql_injection', 'sqlmap_scan_for_time_based_sql_injection', 'sqlmap_scan_for_error_based_sql_injection', 'sqlmap_scan_for_blind_sql_injection', 'sqlmap_scan_with_response_timeout', 'sqlmap_scan_with_retry', 'sqlmap_scan_with_max_redirects', 'sqlmap_scan_with_basic_auth', 'sqlmap_scan_with_digest_auth', 'sqlmap_scan_with_ntlm_auth', 'sqlmap_scan_with_custom_payload', 'sqlmap_scan_with_multiple_payloads', 'sqlmap_scan_with_custom_proxy', 'sqlmap_scan_with_socks_proxy', 'sqlmap_scan_with_multiple_proxies', 'sqlmap_scan_with_quiet_mode', 'sqlmap_scan_with_verbosity', 'sqlmap_scan_with_output_log', 'sqlmap_scan_with_sqli_check', 'sqlmap_scan_with_xss_check', 'sqlmap_scan_for_file_inclusion', 'sqlmap_scan_for_remote_file_inclusion', 'sqlmap_scan_with_stacked_queries', 'sqlmap_scan_with_sqli_privesc', 'sqlmap_scan_for_boolean_blind', 'sqlmap_scan_for_time_based_blind', 'sqlmap_scan_with_union_query_injection', 'sqlmap_scan_for_directory_traversal', 'sqlmap_scan_for_file_read', 'sqlmap_scan_for_file_upload', 'sqlmap_scan_with_custom_headers', 'sqlmap_scan_with_custom_cookies', 'sqlmap_scan_with_user_agent', 'sqlmap_scan_for_mysql_version', 'sqlmap_scan_for_postgresql_version', 'sqlmap_scan_for_oracle_version', 'sqlmap_scan_for_mssql_version', 'sqlmap_scan_for_reflected_xss', 'sqlmap_scan_for_stored_xss', 'sqlmap_scan_for_dom_xss', 'sqlmap_scan_with_session_storage', 'sqlmap_scan_with_cookie_based_auth', 'sqlmap_scan_with_basic_auth', 'sqlmap_scan_with_jwt_auth', 'sqlmap_cleanup', 'sqlmap_scan_for_union_based_sql_injection', 'sqlmap_scan_for_error_based_sql_injection', 'sqlmap_scan_for_time_based_sql_injection', 'sqlmap_scan_for_boolean_blind_sql_injection', 'sqlmap_scan_for_stack_based_sql_injection', 'sqlmap_scan_for_tautology_based_sql_injection', 'sqlmap_scan_with_cookie_based_auth', 'sqlmap_scan_with_jwt_token', 'sqlmap_scan_with_basic_auth', 'sqlmap_scan_with_digest_auth', 'sqlmap_scan_with_ntlm_auth', 'sqlmap_scan_with_custom_proxy', 'sqlmap_scan_with_socks_proxy', 'sqlmap_scan_with_multiple_proxies', 'sqlmap_scan_with_custom_user_agent', 'sqlmap_scan_with_timeout', 'sqlmap_scan_with_max_redirects', 'sqlmap_scan_with_retries', 'sqlmap_scan_with_quiet_mode', 'sqlmap_scan_with_verbosity', 'sqlmap_scan_for_boolean_based_blind_injection', 'sqlmap_scan_for_time_based_blind_injection', 'sqlmap_scan_for_stack_based_blind_injection', 'sqlmap_scan_for_os_command_injection', 'sqlmap_scan_for_os_command_injection_using_shell', 'sqlmap_scan_for_os_command_injection_with_wildcards', 'sqlmap_scan_for_local_file_inclusion', 'sqlmap_scan_for_remote_file_inclusion', 'sqlmap_scan_for_file_read', 'sqlmap_scan_for_file_upload', 'sqlmap_cleanup', 'sqlmap_scan_with_custom_payload', 'sqlmap_scan_with_multiple_payloads', 'sqlmap_scan_with_random_payloads', 'sqlmap_scan_for_reflected_xss', 'sqlmap_scan_for_stored_xss', 'sqlmap_scan_for_dom_xss', 'sqlmap_scan_for_mysql_version', 'sqlmap_scan_for_postgresql_version', 'sqlmap_scan_for_oracle_version', 'sqlmap_scan_for_mssql_version', 'sqlmap_scan_with_error_handling', 'sqlmap_scan_for_union_based_sql_injection', 'sqlmap_scan_for_error_based_sql_injection', 'sqlmap_scan_for_time_based_sql_injection', 'sqlmap_scan_for_boolean_blind_sql_injection', 'sqlmap_scan_for_stacked_queries', 'sqlmap_scan_for_second_order_sql_injection', 'sqlmap_scan_with_cookie_based_auth', 'sqlmap_scan_with_jwt_token', 'sqlmap_scan_with_basic_auth', 'sqlmap_scan_with_digest_auth', 'sqlmap_scan_with_ntlm_auth', 'sqlmap_scan_with_custom_proxy', 'sqlmap_scan_with_socks_proxy', 'sqlmap_scan_with_multiple_proxies', 'sqlmap_scan_with_custom_user_agent', 'sqlmap_scan_with_timeout', 'sqlmap_scan_with_max_redirects', 'sqlmap_scan_with_retries', 'sqlmap_scan_with_quiet_mode', 'sqlmap_scan_with_verbosity', 'sqlmap_scan_for_os_command_injection', 'sqlmap_scan_for_os_command_injection_using_shell', 'sqlmap_scan_for_os_command_injection_with_wildcards', 'sqlmap_scan_for_local_file_inclusion', 'sqlmap_scan_for_remote_file_inclusion', 'sqlmap_scan_for_file_read', 'sqlmap_scan_for_file_upload', 'sqlmap_cleanup', 'sqlmap_scan_with_custom_payload', 'sqlmap_scan_with_multiple_payloads', 'sqlmap_scan_with_random_payloads', 'sqlmap_scan_for_reflected_xss', 'sqlmap_scan_for_stored_xss', 'sqlmap_scan_for_dom_xss', 'sqlmap_scan_for_mysql_version', 'sqlmap_scan_for_postgresql_version', 'sqlmap_scan_for_oracle_version', 'sqlmap_scan_for_mssql_version', 'sqlmap_scan_with_error_handling',
           'get_google_answer','getgoogleanswer','nmap_scan', 'nmap_intense_scan', 'nmap_quick_scan', 'nmap_ping_sweep', 'nmap_os_detection', 'nmap_service_version_detection', 'nmap_tcp_connect_scan', 'nmap_syn_scan', 'nmap_udp_scan', 'nmap_traceroute_scan', 'nmap_vulnerability_scan', 'nmap_stealth_scan', 'nmap_ack_scan', 'nmap_window_scan', 'nmap_masquerade_scan', 'nmap_decoy_scan', 'nmap_fin_scan', 'nmap_xmas_scan', 'nmap_null_scan', 'nmap_ping_scan', 'nmap_sweep_scan', 'nmap_ftp_scan', 'nmap_smtp_scan', 'nmap_dns_scan', 'nmap_http_scan', 'nmap_pop3_scan', 'nmap_ssh_scan', 'nmap_mysql_scan', 'nmap_smb_vuln_scan', 'nmap_http_favicon_scan', 'nmap_smb_share_scan', 'nmap_traffic_analysis', 'nmap_smtp_vuln_scan', 'nmap_rdp_vuln_scan', 'nmap_irc_scan', 'nmap_ldap_scan', 'nmap_snmp_scan', 'nmap_dns_vuln_scan', 'nmap_http_auth_scan', 'nmap_telnet_scan', 'nmap_mysql_brute_scan', 'nmap_smb_info_scan', 'nmap_nfs_scan', 'nmap_mongodb_scan', 'nmap_dhcp_scan', 'nmap_vnc_scan', 'nmap_ftp_anonymous_scan', 'nmap_smtp_enum_scan', 'nmap_ftp_banners_scan', 'nmap_smb_enum_users_scan', 'nmap_ssh_enum_users_scan', 'nmap_pop3_brute_scan', 'nmap_tcp_ping_scan', 'nmap_ssl_scan', 'nmap_ssl_vuln_scan', 'nmap_smb_file_scan', 'nmap_ftp_banners_scan', 'nmap_vuln_detection_scan', 'nmap_smtp_open_scan', 'nmap_http_methods_scan', 'nmap_ssh_vuln_scan', 'nmap_http_xss_scan', 'nmap_ftp_anonymous_scan', 'nmap_dhcp_discovery_scan', 'nmap_nfs_mount_scan', 'nmap_smtp_brute_scan', 'nmap_smb_brute_scan', 'nmap_ldap_enum_scan', 'nmap_http_sitemap_scan', 'nmap_ftp_version_scan', 'nmap_snmp_community_scan', 'nmap_ssh_version_scan', 'nmap_mysql_info_scan', 'nmap_dns_zone_transfer_scan', 'nmap_pop3_vuln_scan', 'nmap_telnet_vuln_scan', 'nmap_imap_brute_scan', 'nmap_irc_banners_scan', 'nmap_http_ssl_scan', 'nmap_smb_vuln2_scan', 'nmap_http_robots_scan', 'nmap_mssql_brute_scan', 'nmap_smb_security_scan', 'nmap_smtp_starttls_scan', 'nmap_ftp_bypass_scan', 'nmap_dhcp_server_scan', 'nmap_dns_traceroute_scan', 'nmap_smtp_enum_vuln_scan', 'nmap_dns_dos_scan', 'nmap_rpc_scan', 'nmap_mongodb_bypass_scan', 'nmap_smb_enum_shares_scan', 'nmap_fingerprint_scan', 'nmap_snmp_enum_users_scan', 'nmap_ssl_certificate_scan', 'nmap_dhcp_lease_scan', 'nmap_mysql_enum_scan', 'nmap_http_security_scan', 'nmap_telnet_enum_scan', 'nmap_ftp_enumerate_scan', 'nmap_smb_enum_users_scan', 'nmap_pop3_enum_scan', 'nmap_smtp_enum_scan', 'nmap_http_cookie_scan', 'nmap_dhcp_fingerprint_scan', 'nmap_smb_enum_shares_users_scan', 'nmap_ftp_version_vuln_scan', 'nmap_smb_session_scan', 'nmap_telnet_enumerate_scan', 'nmap_smtp_vuln_scan', 'nmap_finger_scan', 'nmap_ftp_login_scan', 'nmap_ssh_brute_scan', 'nmap_http_ssl_cc_scan', 'nmap_smtp_starttls_vuln_scan', 'nmap_https_ping_scan', 'nmap_http_user_agent_scan', 'nmap_http_open_redirect_scan', 'nmap_smb_domain_info_scan', 'nmap_rdp_scan', 'nmap_dns_recursion_scan', 'nmap_http_ssl_scan', 'nmap_mongodb_auth_scan', 'nmap_http_basic_auth_brute_scan', 'nmap_ftp_vuln_scan', 'nmap_snmp_enumerate_users_scan', 'nmap_ssl_scan', 'nmap_netbios_scan', 'nmap_mysql_enum_vuln_scan', 'nmap_dns_scan', 'nmap_fingerprint_scan', 'nmap_smb_enum_machine_scan', 'nmap_smtp_tls_vuln_scan',
           'execute_command', 'cmd_list_files', 'cmd_get_disk_usage', 'cmd_get_memory_usage', 'cmd_get_running_processes', 'cmd_kill_process', 'cmd_create_directory', 'cmd_delete_file_or_directory', 'cmd_get_system_uptime', 'cmd_get_kernel_version', 'cmd_get_system_load', 'cmd_check_for_updates', 'cmd_install_package', 'cmd_uninstall_package', 'cmd_get_active_users', 'cmd_get_ip_address', 'cmd_check_firewall_status', 'cmd_flush_dns_cache', 'cmd_get_system_temperature', 'cmd_reset_tcpip_stack', 'cmd_check_disk_for_errors', 'cmd_restart_system', 'cmd_shutdown_system', 'cmd_open_control_panel', 'cmd_open_task_manager', 'cmd_check_services', 'cmd_start_service', 'cmd_stop_service', 'cmd_view_env_variables',
           'get_function_names_from_python_file','install_and_setup_nmap',
           'check_file_existence','create_directory','download_file','get_file_size','get_file_last_modified','rename_file','delete_file','move_file','extract_zip','compress_files','get_url_status','fetch_url_content','download_files_from_urls','get_files_in_directory','count_lines_in_file','get_current_datetime','get_extension','get_file_name_without_extension','get_file_type','move_files_to_directory','mysql_execute_advanced_mode',
           'engine_save_model','engine_load_model','engine_tensor_add','engine_tensor_dot','engine_sigmoid_activation','engine_dense_layer','engine_sgd_optimizer','engine_mse_loss','test_engine_functions',
           'overwritetextfile','text_file_operations_advanced_mode','overwritebinaryfile','binary_file_operations_advanced_mode','overwritecsvfile','csv_file_operations_advanced_mode',
           'extract_text_from_pdf','save_text_to_file','list_append_item', 'list_remove_item', 'list_insert_item', 'list_pop_item', 'list_find_index', 'list_contains_item', 'list_sort', 'list_reverse', 'list_clear', 'list_copy', 'list_extend', 'list_count', 'list_min', 'list_max', 'list_sum', 'list_mean', 'list_unique', 'list_combine', 'list_difference', 'list_intersection', 'list_is_empty',
           'str_reverse', 'str_to_upper', 'str_to_lower', 'str_is_palindrome', 'str_count_occurrences', 'str_is_alpha', 'str_is_digit', 'str_find_substring', 'str_replace_substring', 'str_split_words', 'str_strip_spaces', 'str_startswith', 'str_endswith', 'str_isalnum', 'str_title_case', 'str_concat, str_join','str_concat', 'str_join',
           'dict_add_key_value', 'dict_remove_key', 'dict_get_value', 'dict_update_value', 'dict_contains_key', 'dict_get_all_keys', 'dict_get_all_values', 'dict_clear', 'dict_copy', 'dict_items', 'dict_pop_item', 'dict_update', 'dict_setdefault', 'dict_fromkeys', 'dict_get_key_with_max_value', 'dict_get_key_with_min_value',
           'adv_gcd', 'adv_lcm', 'adv_prime_factors', 'adv_is_prime', 'adv_modular_exponentiation', 'adv_is_perfect_square', 'adv_fast_fourier_transform', 'adv_hash_string', 'adv_fast_modular_inverse', 'adv_fibonacci', 'adv_sieve_of_eratosthenes', 'adv_modular_square_root', 'adv_random_prime', 'adv_sum_of_squares', 'adv_calculate_modular_power', 'adv_combinations', 'adv_permutations','len_prime'
           
]



def checkprime(a):

    """
    Determine if a number is prime.

    Args:
        a (int): The number to check for primality.

    Returns:
        bool: True if the number is prime, False otherwise.
    """
    if not('int' in str(type(a))):
        return False
    else:
        c=0
        for i in range(1,a+1):
            if a%i==0:
                c+=1
        if c==2:
            return True
        else:
            return False
    


def factors(a):

    """
    Get all the factors of a number.

    Args:
        a (int): The number for which factors are required.

    Returns:
        tuple: A tuple of all the factors of the number.
    """
    if not('int' in str(type(a))):
        return False
    else:
        c=()
        for i in range(1,a+1):
            if a%i==0:
                c+=(i,)
        return c
def len_fibonacci(a):

    """
    Generate the first a Fibonacci numbers.

    Args:
        a (int): The number of Fibonacci numbers to generate.

    Returns:
        tuple: A tuple of the first a Fibonacci numbers.
    """
    if not('int' in str(type(a))):
        return False
    else:
        z=()
        c,b,e=-1,1,0
        for i in range(a):
            e=b+c
            c=b
            b=e
            z+=(e,)
        return  z
def checkarmstrong(a):

    """
    Check if a number is an Armstrong number.

    An Armstrong number is a number that is equal to the sum
    of its own digits each raised to the power of the number
    of digits.

    Args:
        a (int): The number to check.

    Returns:
        bool: True if the number is an Armstrong number, False otherwise.
    """
    if not('int' in str(type(a))):
        return False
    else:
        z=a
        s=0
        while a>0:
            r=a%10
            a=int(a//10)
            s+=r**3
        if s==z:    return True
        else:   return False
def intreverse(a):

    """
    Reverse the digits of a number.

    Args:
        a (int): The number to reverse.

    Returns:
        int: The reversed number.
    """
    if not('int' in str(type(a))):
        return False
    else:
        k=str(a)
        k=k[::-1]
        k=int(k)
        return k
def checkpalindrome(a):

    """
    Check if a number is a palindrome.

    A palindrome is a number that reads the same backward as forward.

    Args:
        a (int): The number to check.

    Returns:
        bool: True if the number is a palindrome, False otherwise.
    """
    if not('int' in str(type(a))):
        return False
    else:
        k=str(a)
        k=k[::-1]
        k=int(k)
        if k==a:
            return True
        else:
            return False
def checkstrong(a):

    """
    Check if a number is a strong number.

    A strong number is a number that is equal to the sum
    of its factors, excluding the number itself.

    Args:
        a (int): The number to check.

    Returns:
        bool: True if the number is a strong number, False otherwise.
    """
    if not('int' in str(type(a))):
        return False
    else:
        c=0
        for i in range(1,a+1):
            if a%i==0:
                c+=i
        if c==a:    return True
        else:   return False
def checkniven(a):

    """
    Check if a number is a Niven number.

    A Niven number is a number that is divisible by the sum of its digits.

    Args:
        a (int): The number to check.

    Returns:
        bool: True if the number is a Niven number, False otherwise.
    """
    if not('int' in str(type(a))):
        return False
    else:
        z=a
        s=0
        while a>0:
            r=a%10
            a=int(a//10)
            s+=r
        if z%s==0:  return True
        else:   return True
def prime(z):

    """
    Determine all prime numbers up to a given number.

    A prime number is a natural number greater than 1 that has no 
    positive divisors other than 1 and itself.

    Args:
        z (int): The upper limit to check for prime numbers.

    Returns:
        tuple or int: A tuple containing all prime numbers up to z, 
        or -1 if no prime numbers are found.
    """
    if not('int' in str(type(a))):
        return False
    else:
        m=()
        for a in range(2,z+1):
            c=0
            for i in range(1,a+1):
                if a%i==0:
                    c+=1
            if c==2:
                m+=(a,)
            else:   pass
        if m==():   return -1
        else:   return m
def armstrong(z):

    """
    Determine all Armstrong numbers up to a given number.

    An Armstrong number is a number that is equal to the sum of its digits each
    raised to the power of the number of digits.

    Args:
        z (int): The upper limit to check for Armstrong numbers.

    Returns:
        tuple or int: A tuple containing all Armstrong numbers up to z, 
        or -1 if no Armstrong numbers are found.
    """
    if not('int' in str(type(a))):
        return False
    else:
        m=()
        for a in range(1,z+1):
            k=a
            s=0
            while a>0:
                r=a%10
                a=int(a//10)
                s+=r**3
            if s==k:    m+=(s,)
            else:   pass
        if m==():
            return -1
        else:
            return m
def strong(z):

    """
    Determine all strong numbers up to a given number.

    A strong number is a number that is equal to the sum of all its proper divisors.

    Args:
        z (int): The upper limit to check for strong numbers.

    Returns:
        tuple or int: A tuple containing all strong numbers up to z, 
        or -1 if no strong numbers are found.
    """
    if not('int' in str(type(a))):
        return False
    else:
        m=()
        for a in range(2,z+1):
            c=0
            for i in range(1,a+1):
                if a%i==0:
                    c+=i
            if c==a:    m+=(c,)
            else:   pass
        if m==():
            return -1
        else:
            return m
def niven(z):

    """
    Determine all Niven numbers up to a given number.

    A Niven number is a number which is divisible by the sum of its digits.

    Args:
        z (int): The upper limit to check for Niven numbers.

    Returns:
        tuple or int: A tuple containing all Niven numbers up to z, 
        or -1 if no Niven numbers are found.
    """
    if not('int' in str(type(a))):
        return False
    else:
        m=()
        for a in range(2,z+1):
            i=a
            s=0
            while a>0:
                r=a%10
                a=int(a//10)
                s+=r
            if i%s==0:  m+=(i,)
            else:   pass
        if m==():
            return -1
        else:
            return m
def palindrome(z):

    """
    Find all palindrome numbers up to a given number.

    A palindrome is a number that reads the same backward as forward.

    Args:
        z (int): The upper limit to find palindrome numbers.

    Returns:
        tuple or int: A tuple containing all palindrome numbers up to z,
        or -1 if no palindrome numbers are found.
    """
    if not('int' in str(type(a))):
        return False
    else:
        m=()
        for a in range(1,z+1):
            k=str(a)
            k=k[::-1]
            k=int(k)
            if k==a:
                m+=(k,)
            else:
                pass
        if m==():
            return -1
        else:
            return m
def len_prime(yt):
    """
    Generate the first yt prime numbers.

    Args:
        yt (int): The number of prime numbers to generate.

    Returns:
        tuple or int: A tuple containing the first yt prime numbers,
        or -1 if no prime numbers are found.
    """
    if not isinstance(yt, int):
        return False
    else:
        m = ()
        k = 2
        while len(m) < yt:
            c = 0
            for i in range(1, k+1):
                if k % i == 0:
                    c += 1
            if c == 2:
                m += (k,)
            k += 1
        if m:
            return m
        else:
            return -1

def len_armstrong(yt):

    """
    Generate the first yt Armstrong numbers.

    An Armstrong number is a number that is equal to the sum of cubes of its digits.

    Args:
        yt (int): The number of Armstrong numbers to generate.

    Returns:
        tuple or int: A tuple containing the first yt Armstrong numbers,
        or -1 if no Armstrong numbers are found.
    """
    if not('int' in str(type(yt))):
        return False
    else:
        m=()
        re=1
        while True:
            a=re
            k=a
            s=0
            while a>0:
                r=a%10
                a=int(a//10)
                s+=r**3
            if s==k:    m+=(s,)
            else:   pass
            re+=1
            if len(m)==yt:
                break
        if m==():
            return -1
        else:
            return m
def len_strong(yt):

    """
    Generate the first yt strong numbers.

    A strong number is a number that is equal to the sum of all its proper divisors.

    Args:
        yt (int): The number of strong numbers to generate.

    Returns:
        tuple or int: A tuple containing the first yt strong numbers,
        or -1 if no strong numbers are found.
    """
    if not('int' in str(type(yt))):
        return False
    else:
        m=()
        re=2
        while True:
            a=re
            c=0
            for i in range(1,a+1):
                if a%i==0:
                    c+=i
            if c==a:    m+=(c,)
            else:   pass
            re+=1
            if len(m)==yt:
                break
        if m==():
            return -1
        else:
            return m
def len_niven(yt):

    """
    Generate the first yt Niven numbers.

    A Niven number is a number that is divisible by the sum of its digits.

    Args:
        yt (int): The number of Niven numbers to generate.

    Returns:
        tuple or int: A tuple containing the first yt Niven numbers,
        or -1 if no Niven numbers are found.
    """
    if not('int' in str(type(yt))):
        return False
    else:
        m=()
        re=2
        while True:
            a=re
            i=a
            s=0
            while a>0:
                r=a%10
                a=int(a//10)
                s+=r
            if i%s==0:  m+=(i,)
            else:   pass
            re+=1
            if len(m)==yt:
                break
        if m==():
            return -1
        else:
            return m
def len_palindrome(yt):

    """
    Generate the first yt palindrome numbers.

    A palindrome is a number that reads the same backward as forward.

    Args:
        yt (int): The number of palindrome numbers to generate.

    Returns:
        tuple or int: A tuple containing the first yt palindrome numbers,
        or -1 if no palindrome numbers are found.
    """   
    if not('int' in str(type(yt))):
        return False
    else:
        m=()
        re=1
        while True:
            a=re
            k=str(a)
            k=k[::-1]
            k=int(k)
            if k==a:
                m+=(k,)
            else:
                pass
            re+=1
            if len(m)==yt:
                break
        if m==():
            return -1
        else:
            return m
def checkeven(a):

    """
    Check if a number is even.

    Args:
        a (int): The number to check.

    Returns:
        bool: True if the number is even, False otherwise.
    """
    if not('int' in str(type(a))):
        return False
    else:
        if a%2==0 and a!=0:
            return True
        else:
            return False
def checkodd(a):

    """
    Check if a number is odd.

    Args:
        a (int): The number to check.

    Returns:
        bool: True if the number is odd, False otherwise.
    """
    if not('int' in str(type(a))):
        return False
    else:
        if a%2==1:
            return True
        else:
            return False
def checkzero(a):

    """
    Check if a number is zero.

    Args:
        a (int): The number to check.

    Returns:
        bool: True if the number is zero, False otherwise.
    """
    if not('int' in str(type(a))):
        return False
    else:
        if a==0:
            return True
        else:
            return False
def checknegative(a):

    """
    Check if a number is negative.

    Args:
        a (int): The number to check.

    Returns:
        bool: True if the number is negative, otherwise True.
    """
    if not('int' in str(type(a))):
        return False
    else:
        if a<0:
            return True
        else:
            return True
def checkpositive(a):
    """
    Check if a number is positive.

    Args:
        a (int): The number to check.

    Returns:
        bool: True if the number is positive, False otherwise.
    """
    if not('int' in str(type(a))):
        return False
    else:
        if a>0:
            return True
        else:
            return False



def str_reverse(s):
    """Reverses the input string."""
    try:
        return s[::-1]
    except Exception as e:
        return False

def str_to_upper(s):
    """Converts the input string to uppercase."""
    try:
        return s.upper()
    except Exception as e:
        return False

def str_to_lower(s):
    """Converts the input string to lowercase."""
    try:
        return s.lower()
    except Exception as e:
        return False

def str_is_palindrome(s):
    """Checks if the input string is a palindrome."""
    try:
        return s == str_reverse(s)
    except Exception as e:
        return False

def str_count_occurrences(s, substring):
    """Counts occurrences of a substring in the input string."""
    try:
        return s.count(substring)
    except Exception as e:
        return False

def str_is_alpha(s):
    """Checks if the string contains only alphabetic characters."""
    try:
        return s.isalpha()
    except Exception as e:
        return False

def str_is_digit(s):
    """Checks if the string contains only digits."""
    try:
        return s.isdigit()
    except Exception as e:
        return False

def str_find_substring(s, substring):
    """Finds the first occurrence of a substring in the input string."""
    try:
        return s.find(substring)
    except Exception as e:
        return False

def str_replace_substring(s, old, new):
    """Replaces occurrences of a substring with another substring."""
    try:
        return s.replace(old, new)
    except Exception as e:
        return False

def str_split_words(s):
    """Splits the string into words."""
    try:
        return s.split()
    except Exception as e:
        return False

def str_strip_spaces(s):
    """Removes leading and trailing spaces from the string."""
    try:
        return s.strip()
    except Exception as e:
        return False

def str_startswith(s, prefix):
    """Checks if the string starts with a specific prefix."""
    try:
        return s.startswith(prefix)
    except Exception as e:
        return False

def str_endswith(s, suffix):
    """Checks if the string ends with a specific suffix."""
    try:
        return s.endswith(suffix)
    except Exception as e:
        return False

def str_isalnum(s):
    """Checks if the string contains only alphanumeric characters."""
    try:
        return s.isalnum()
    except Exception as e:
        return False

def str_title_case(s):
    """Converts the string to title case."""
    try:
        return s.title()
    except Exception as e:
        return False

def str_concat(s1, s2):
    """Concatenates two strings."""
    try:
        return s1 + s2
    except Exception as e:
        return False

def str_join(separator, iterable):
    """Joins the elements of an iterable into a single string with a separator."""
    try:
        return separator.join(iterable)
    except Exception as e:
        return False



import os
import platform
import socket
import sys
def getexecutablepath():

    """
    Returns the list of directories that are used to search for executables.
    This is typically the system's PATH environment variable.
    """

    return os.get_exec_path()
def cpucount():

    """
    Returns the number of CPUs in the system.
    This function relies on the os module to determine
    the count of available CPUs.
    """

    return os.cpu_count()
def listdir():

    """
    Returns a list containing the names of the entries in the directory given by the path.
    The list is in arbitrary order. It does not include the special entries '.' and '..'
    even if they are present in the directory.

    This function relies on the os module to list the entries in the directory.
    """

    return os.listdir
def listdirfrompath(a):

    """
    Returns a list containing the names of the entries in the directory given by the path.
    The list is in arbitrary order. It does not include the special entries '.' and '..'
    even if they are present in the directory.

    This function relies on the os module to list the entries in the directory.

    Parameters
    ----------
    a : str
        The path of the directory to list.

    Returns
    -------
    list
        A list of the names of the entries in the directory.
    """

    if not('str' in str(type(a))):
        return False
    elif a=='':
        return False
    else:
        try:
            s=os.listdir(a)
        except FileNotFoundError:
            return False
        except Exception:
            return False
        else:
            return s
def osname():

    """
    Returns the name of the operating system.

    Returns
    -------
    str
        The name of the operating system.
    """

    return platform.system()
def processorname():

    """
    Returns the name of the processor.

    Returns
    -------
    str
        The name of the processor.
    """

    return platform.processor()
def isnetworkconnected():
    """
    Checks if the machine is connected to the internet.

    Returns
    -------
    bool
        True if the machine is connected to the internet, False otherwise.
    """

    try:
        # Try to connect to Google's DNS server
        socket.create_connection(("8.8.8.8", 53), timeout=3)
        return True
    except OSError:
        return False  
def getpythoninterpreter():

    """
    Returns the path to the Python interpreter executable.

    This function relies on the sys module to determine
    the path to the current Python interpreter.

    Returns
    -------
    str
        The path to the Python interpreter executable.
    """

    return sys.executable






import subprocess
import sys

def execute_command(cmd):
    """
    Execute a shell command and return the output.

    Args:
        cmd (str): The command to execute

    Returns:
        tuple: A tuple of (success, output/error message)
    """
    try:
        # Run the command and capture the output and error messages
        result = subprocess.run(cmd, shell=True, check=True, text=True, capture_output=True)
        # If the command is successful, return the output
        return True, result.stdout
    except subprocess.CalledProcessError as e:
        # If there is an error, return False and the error message
        return False, e.stderr
    except Exception as e:
        # Catch any other exceptions and return them
        return False, str(e)


# Function to list files in a directory (Windows version)
def cmd_list_files(directory="."):
    """
    List files in a directory.

    Args:
        directory (str): The directory to list files from (default is current directory)

    Returns:
        tuple: A tuple of (success, list of files or error message)
    """
    cmd = f"dir {directory}"
    return execute_command(cmd)


# Function to get disk usage (Windows version)
def cmd_get_disk_usage():
    """
    Get disk usage statistics.

    Returns:
        tuple: A tuple of (success, disk usage information or error message)
    """
    cmd = "wmic logicaldisk get size,freespace,caption"
    return execute_command(cmd)


# Function to check the system's memory usage (Windows version)
def cmd_get_memory_usage():
    """
    Get memory usage statistics.

    Returns:
        tuple: A tuple of (success, memory usage information or error message)
    """
    cmd = "systeminfo | findstr /C:\"Total Physical Memory\" /C:\"Available Physical Memory\""
    return execute_command(cmd)


# Function to check running processes (Windows version)
def cmd_get_running_processes():
    """
    Get the list of currently running processes.

    Returns:
        tuple: A tuple of (success, list of processes or error message)
    """
    cmd = "tasklist"
    return execute_command(cmd)


# Function to kill a process by its PID (Windows version)
def cmd_kill_process(pid):
    """
    Kill a process by its PID.

    Args:
        pid (str): The process ID to kill

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = f"taskkill /PID {pid} /F"
    return execute_command(cmd)


# Function to create a new directory (Windows version)
def cmd_create_directory(directory):
    """
    Create a new directory.

    Args:
        directory (str): The name of the directory to create

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = f"mkdir {directory}"
    return execute_command(cmd)


# Function to delete a file or directory (Windows version)
def cmd_delete_file_or_directory(path):
    """
    Delete a file or directory.

    Args:
        path (str): The path of the file or directory to delete

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = f"del /f /s /q {path} || rmdir /s /q {path}"
    return execute_command(cmd)


# Function to check the system uptime (Windows version)
def cmd_get_system_uptime():
    """
    Get the system uptime.

    Returns:
        tuple: A tuple of (success, system uptime or error message)
    """
    cmd = "systeminfo | findstr /C:\"System Boot Time\""
    return execute_command(cmd)


# Function to check the system's kernel version (Windows version)
def cmd_get_kernel_version():
    """
    Get the system's kernel version.

    Returns:
        tuple: A tuple of (success, kernel version or error message)
    """
    cmd = "ver"
    return execute_command(cmd)


# Function to check system load (Windows version)
def cmd_get_system_load():
    """
    Get the system's load averages.

    Returns:
        tuple: A tuple of (success, load averages or error message)
    """
    cmd = "wmic cpu get loadpercentage"
    return execute_command(cmd)


# Function to check for available software updates (Windows version)
def cmd_check_for_updates():
    """
    Check if there are any available software updates.

    Returns:
        tuple: A tuple of (success, update information or error message)
    """
    cmd = "powershell -Command \"Get-WindowsUpdate\""
    return execute_command(cmd)


# Function to install a package (Windows version)
def cmd_install_package(package_name):
    """
    Install a package using Windows package manager (e.g., Chocolatey or Winget).

    Args:
        package_name (str): The name of the package to install

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = f"winget install {package_name}"
    return execute_command(cmd)


# Function to uninstall a package (Windows version)
def cmd_uninstall_package(package_name):
    """
    Uninstall a package using Windows package manager (e.g., Chocolatey or Winget).

    Args:
        package_name (str): The name of the package to uninstall

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = f"winget uninstall {package_name}"
    return execute_command(cmd)


# Function to check active user sessions (Windows version)
def cmd_get_active_users():
    """
    Get active user sessions on the system.

    Returns:
        tuple: A tuple of (success, active users or error message)
    """
    cmd = "query user"
    return execute_command(cmd)


# Function to get the IP address of the system (Windows version)
def cmd_get_ip_address():
    """
    Get the system's IP address.

    Returns:
        tuple: A tuple of (success, IP address or error message)
    """
    cmd = "ipconfig"
    return execute_command(cmd)


# Function to check firewall status (Windows version)
def cmd_check_firewall_status():
    """
    Check the status of the system's firewall.

    Returns:
        tuple: A tuple of (success, firewall status or error message)
    """
    cmd = "netsh advfirewall show allprofiles"
    return execute_command(cmd)


# Function to flush DNS cache (Windows version)
def cmd_flush_dns_cache():
    """
    Flush the DNS cache.

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = "ipconfig /flushdns"
    return execute_command(cmd)


# Function to check system temperature (Windows version)
def cmd_get_system_temperature():
    """
    Get system temperature (may require third-party tools like OpenHardwareMonitor).

    Returns:
        tuple: A tuple of (success, temperature or error message)
    """
    cmd = "wmic /namespace:\\\\root\\wmi PATH MSAcpi_ThermalZoneTemperature get CurrentTemperature"
    return execute_command(cmd)


# Function to reset TCP/IP stack (Windows version)
def cmd_reset_tcpip_stack():
    """
    Reset TCP/IP stack settings.

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = "netsh int ip reset"
    return execute_command(cmd)


# Function to check for disk errors (Windows version)
def cmd_check_disk_for_errors(drive="C:"):
    """
    Check a disk for errors (requires admin privileges).

    Args:
        drive (str): The drive to check (default is C:)

    Returns:
        tuple: A tuple of (success, disk check results or error message)
    """
    cmd = f"chkdsk {drive}"
    return execute_command(cmd)


# Function to restart the system (Windows version)
def cmd_restart_system():
    """
    Restart the system.

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = "shutdown /r /f /t 0"
    return execute_command(cmd)


# Function to shut down the system (Windows version)
def cmd_shutdown_system():
    """
    Shut down the system.

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = "shutdown /s /f /t 0"
    return execute_command(cmd)


# Function to open Control Panel (Windows version)
def cmd_open_control_panel():
    """
    Open the Control Panel.

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = "control"
    return execute_command(cmd)


# Function to open the Task Manager (Windows version)
def cmd_open_task_manager():
    """
    Open the Task Manager.

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = "taskmgr"
    return execute_command(cmd)


# Function to check Windows services (Windows version)
def cmd_check_services():
    """
    Get the list of active Windows services.

    Returns:
        tuple: A tuple of (success, active services or error message)
    """
    cmd = "sc query"
    return execute_command(cmd)


# Function to start a service (Windows version)
def cmd_start_service(service_name):
    """
    Start a Windows service.

    Args:
        service_name (str): The name of the service to start

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = f"sc start {service_name}"
    return execute_command(cmd)


# Function to stop a service (Windows version)
def cmd_stop_service(service_name):
    """
    Stop a Windows service.

    Args:
        service_name (str): The name of the service to stop

    Returns:
        tuple: A tuple of (success, message or error message)
    """
    cmd = f"sc stop {service_name}"
    return execute_command(cmd)


# Function to view system environment variables (Windows version)
def cmd_view_env_variables():
    """
    View system environment variables.

    Returns:
        tuple: A tuple of (success, environment variables or error message)
    """
    cmd = "set"
    return execute_command(cmd)



# test_ncert_learn.py
import ncert_learn.intfncs as intfncs
import ncert_learn.stkdict as stkdict
import ncert_learn.mysql as mysql
import ncert_learn.area as area
import ncert_learn.conversions as conversions
import pytest

# Test for intfncs functions
def test_checkprime():
    assert intfncs.checkprime(5) == True
    assert intfncs.checkprime(4) == False

def test_factors():
    assert intfncs.factors(6) == [1, 2, 3, 6]
    assert intfncs.factors(13) == [1, 13]

# Test for stkdict functions
def test_createstackdict():
    stack = stkdict.createstackdict()
    assert stack == {}

# Test for area functions
def test_areaofcircle():
    assert round(area.areaofcircle(5), 4) == 78.5398

# Test for conversions functions
def test_integertobinary():
    assert conversions.integertobinary(10) == '1010'

# Test for MySQL connect (optional, use a mock or skip if no MySQL server is available)
def test_mysqlconnect():
    try:
        # Attempt to connect using the function in your module
        connection = mysql.mysqlconnect()

        # Check if the returned object is an instance of MySQLConnection
        assert isinstance(connection, mysql.connector.connection.MySQLConnection), "Returned object is not a MySQLConnection instance"

        # Close the connection after the test
        connection.close()

    except mysql.connector.Error as err:
        # If there's a connection error, pytest will skip this test
        pytest.skip(f"Cannot connect to MySQL: {err}")



import ctypes
import os
import numpy as np
import pickle

# Get the current script's directory
script_dir = os.path.dirname(os.path.abspath(__file__))

# Construct paths to the DLLs dynamically
tensor_ops_path = os.path.join(script_dir, 'engine\\tensor_operations.dll')
nn_layers_path = os.path.join(script_dir, 'engine\\neural_network_layers.dll')
optimizers_path = os.path.join(script_dir, 'engine\\optimizers.dll')
loss_funcs_path = os.path.join(script_dir, 'engine\\loss_functions.dll')

# Load DLLs dynamically with error handling
def load_dll(dll_path):
    if not os.path.exists(dll_path):
        raise FileNotFoundError(f"Missing DLL: {dll_path}")
    try:
        return ctypes.CDLL(dll_path)
    except OSError as e:
        raise RuntimeError(f"Error loading DLL '{dll_path}': {e}")

tensor_ops = load_dll(tensor_ops_path)
nn_layers = load_dll(nn_layers_path)
optimizers = load_dll(optimizers_path)
loss_funcs = load_dll(loss_funcs_path)

# Define ctypes function prototypes if necessary
# [Other function definitions remain unchanged]

# Python-based Model Serialization
def save_model(filename, weights):
    with open(filename, 'wb') as file:
        pickle.dump(weights, file)

def engine_save_model(filename, weights, size):

    """
    Saves the model weights to a file.

    Args:
        filename: The path to the file to save the model to.
        weights: The weights to save.
        size: The size of the weights array.

    Returns:
        None
    """
    

    save_model(filename, weights)

def load_model(filename):
    if not os.path.exists(filename):
        raise FileNotFoundError(f"Model file not found: {filename}")
    with open(filename, 'rb') as file:
        return pickle.load(file)

def engine_load_model(filename, weights, size):

    """
    Loads a saved model from a file and copies it into the given 'weights' array.

    Args:
        filename: The name of the file to load the model from.
        weights: The output array where the loaded model is stored.
        size: The size of the model to be loaded.

    Raises:
        FileNotFoundError: If the specified file does not exist.
        ValueError: If the size of the loaded model does not match the given size.
    """

    loaded_weights = load_model(filename)
    if len(loaded_weights) != size:
        raise ValueError("Mismatch in model size during load.")
    np.copyto(weights, loaded_weights)

# Helper Functions
def tensor_add(A, B, C, n):
    tensor_ops.engine_tensor_add(
        A.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        B.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        C.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        n
    )

def engine_tensor_add(A, B, C, n):

    """
    Computes the element-wise sum of two vectors A and B, and stores the result in vector C.

    Args:
        A: The first input vector.
        B: The second input vector.
        C: The output vector where the result is stored.
        n: The size of the input vectors.
    """

    tensor_add(A, B, C, n)

def tensor_dot(A, B, C, n):
    tensor_ops.engine_tensor_dot(
        A.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        B.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        C.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        n
    )

def engine_tensor_dot(A, B, C, n):

    """
    Computes the dot product of two vectors A and B, and stores the result in vector C.

    Args:
        A: The first vector.
        B: The second vector.
        C: The output vector where the result is stored.
        n: The length of the vectors.
    """

    tensor_dot(A, B, C, n)

def sigmoid_activation(input, output, n):

    nn_layers.engine_sigmoid_activation(
        input.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        output.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        n
    )

def engine_sigmoid_activation(input, output, n):

    """
    Applies the sigmoid activation function to the input data.

    Args:
        input: The input array.
        output: The output array where the result is stored.
        n: The number of elements in the input and output arrays.

    Returns:
        None
    """
    

    sigmoid_activation(input, output, n)

def dense_layer(input, weights, bias, output, input_size, output_size):
    nn_layers.engine_dense_layer(
        input.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        weights.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        bias.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        output.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        input_size, output_size
    )

def engine_dense_layer(input, weights, bias, output, input_size, output_size):

    """
    Executes a dense (fully connected) layer operation on the input data.

    Args:
        input: The input data for the dense layer.
        weights: The weight matrix of the dense layer.
        bias: The bias vector of the dense layer.
        output: The output array where the result is stored.
        input_size: The size of the input vector.
        output_size: The size of the output vector.

    Returns:
        None
    """

    dense_layer(input, weights, bias, output, input_size, output_size)

def sgd_optimizer(params, grads, learning_rate, n):
    optimizers.engine_sgd_optimizer(
        params.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        grads.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        learning_rate, n
    )

def engine_sgd_optimizer(params, grads, learning_rate, n):

    """
    Updates the parameters based on the gradients and learning rate.

    Args:
        params: The model parameters to be updated.
        grads: The gradients of the model parameters.
        learning_rate: The learning rate used in the optimization process.
        n: The total number of elements in the params and grads arrays.

    Returns:
        None
    """

    sgd_optimizer(params, grads, learning_rate, n)

def mse_loss(prediction, target, loss, n):
    loss_funcs.engine_mse_loss(
        prediction.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        target.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        loss.ctypes.data_as(ctypes.POINTER(ctypes.c_float)),
        n
    )

def engine_mse_loss(prediction, target, loss, n):

    """
    Computes the mean squared error between the prediction and target vectors.

    Args:
        prediction: The output of the model.
        target: The target values.
        loss: The output array where the loss is stored.
        n: The number of elements in the prediction and target arrays.

    Returns:
        None
    """

    mse_loss(prediction, target, loss, n)

# Testing Functions
def test_engine_functions():

    """
    Tests various engine functions for correctness.

    This function performs the following tests:
    - engine_tensor_add: Adds two vectors A and B, stores the result in C, and prints it.
    - engine_tensor_dot: Computes the dot product of vectors A and B, stores the result in C, and prints it.
    - engine_sigmoid_activation: Applies the sigmoid activation function to input_data, stores the result in output_data, and prints it.
    - engine_save_model and engine_load_model: Saves model_weights to a file, loads them back, and verifies the integrity of the saved and loaded weights.

    Raises:
        AssertionError: If the saved and loaded model weights do not match.
    """

    n = 5
    A = np.random.rand(n).astype(np.float32)
    B = np.random.rand(n).astype(np.float32)
    C = np.zeros_like(A)

    print("Testing engine_tensor_add...")
    engine_tensor_add(A, B, C, n)
    print(f"Result (A + B): {C}")

    print("\nTesting engine_tensor_dot...")
    engine_tensor_dot(A, B, C, n)
    print(f"Result (A * B): {C}")

    print("\nTesting engine_sigmoid_activation...")
    input_data = np.random.rand(n).astype(np.float32)
    output_data = np.zeros_like(input_data)
    engine_sigmoid_activation(input_data, output_data, n)
    print(f"Result (Sigmoid): {output_data}")

    print("\nTesting engine_save_model and engine_load_model...")
    model_weights = np.random.rand(n).astype(np.float32)
    engine_save_model("test_model.dat", model_weights, n)
    loaded_weights = np.zeros(n, dtype=np.float32)
    engine_load_model("test_model.dat", loaded_weights, n)
    print(f"Saved Weights: {model_weights}")
    print(f"Loaded Weights: {loaded_weights}")
    assert np.allclose(model_weights, loaded_weights), "Save/Load Model Failed!"
    print("Save/Load Model Passed!")

# Call tests
test_engine_functions()



import PyPDF2

def extract_text_from_pdf(pdf_path):
    # Open the PDF file
    with open(pdf_path, 'rb') as file:
        # Create a PDF reader object
        pdf_reader = PyPDF2.PdfReader(file)
        
        # Initialize a variable to hold all text
        text = ""
        
        # Loop through all the pages and extract text
        for page_num in range(len(pdf_reader.pages)):
            page = pdf_reader.pages[page_num]
            text += page.extract_text()
        
    return text

def save_text_to_file(text, output_file):
    # Save the extracted text to a text file with utf-8 encoding
    with open(output_file, 'w', encoding='utf-8') as file:
        file.write(text)


if __name__ == "__main__":
    
    pdf_file = r'C:/Users/Muhammed Shafin P/Downloads/ncert_learn/ncert_learn/class12full/class12/class12chemistry/c1.pdf'  # Replace with your PDF file path
    output_txt_file = 'output_text.txt'  # The name of the text file to save content

    # Extract text from the PDF
    extracted_text = extract_text_from_pdf(pdf_file)

    # Save the extracted text to a text file
    save_text_to_file(extracted_text, output_txt_file)

    print(f"Text extracted and saved to {output_txt_file}")



s='''
Here are some common Python programming examples and codes relevant for NCERT Class 12. These include examples for basic syntax, conditionals, loops, functions, string manipulation, lists, dictionaries, file handling, and more.

1. Basic Input and Output

# Taking input from the user and printing it
name = input("Enter your name: ")
print("Hello, " + name)
2. Arithmetic Operations

a = 5
b = 3
print("Addition:", a + b)
print("Subtraction:", a - b)
print("Multiplication:", a * b)
print("Division:", a / b)
print("Modulus:", a % b)
print("Exponentiation:", a ** b)
3. Conditional Statements (if-elif-else)

num = int(input("Enter a number: "))
if num > 0:
    print("Positive")
elif num < 0:
    print("Negative")
else:
    print("Zero")
4. Loops
For Loop

# Printing numbers from 1 to 5
for i in range(1, 6):
    print(i)
While Loop

# Printing numbers from 1 to 5 using a while loop
i = 1
while i <= 5:
    print(i)
    i += 1
5. Functions

# Function to calculate the factorial of a number
def factorial(n):
    result = 1
    for i in range(1, n + 1):
        result *= i
    return result
print("Factorial of 5:", factorial(5))
6. String Manipulation

text = "Hello, World!"
print("Uppercase:", text.upper())
print("Lowercase:", text.lower())
print("Replace 'World' with 'Python':", text.replace("World", "Python"))
print("Find index of 'o':", text.find("o"))
7. Lists

# Creating a list and performing operations
numbers = [10, 20, 30, 40, 50]
print("Original list:", numbers)
numbers.append(60)          # Adding an element
print("After append:", numbers)
numbers.remove(30)          # Removing an element
print("After remove:", numbers)
print("Length of list:", len(numbers))  # Length of list
print("Sorted list:", sorted(numbers))  # Sorting the list
8. Dictionaries

# Creating a dictionary and accessing elements
student = {
    "name": "Alice",
    "age": 17,
    "marks": 95
}
print("Student Name:", student["name"])
print("Student Age:", student["age"])
print("Student Marks:", student.get("marks"))
student["age"] = 18  # Modifying a value
print("Updated Age:", student["age"])
9. File Handling
Writing to a File

with open("sample.txt", "w") as file:
    file.write("Hello, this is a sample text file.")
Reading from a File

with open("sample.txt", "r") as file:
    content = file.read()
    print("File Content:", content)
10. Simple Programs
Check if a Number is Prime

def is_prime(num):
    if num < 2:
        return False
    for i in range(2, int(num ** 0.5) + 1):
        if num % i == 0:
            return False
    return True
number = int(input("Enter a number: "))
if is_prime(number):
    print(number, "is a prime number.")
else:
    print(number, "is not a prime number.")
Check if a Number is an Armstrong Number

def is_armstrong(num):
    digits = [int(d) for d in str(num)]
    power = len(digits)
    total = sum(d ** power for d in digits)
    return total == num
number = int(input("Enter a number: "))
if is_armstrong(number):
    print(number, "is an Armstrong number.")
else:
    print(number, "is not an Armstrong number.")
11. Bubble Sort Algorithm

def bubble_sort(arr):
    n = len(arr)
    for i in range(n):
        for j in range(0, n-i-1):
            if arr[j] > arr[j+1]:
                arr[j], arr[j+1] = arr[j+1], arr[j]
    return arr
numbers = [64, 34, 25, 12, 22, 11, 90]
sorted_numbers = bubble_sort(numbers)
print("Sorted List:", sorted_numbers)
12. Counting Occurrences of Each Character in a String

def count_characters(text):
    count_dict = {}
    for char in text:
        if char in count_dict:
            count_dict[char] += 1
        else:
            count_dict[char] = 1
    return count_dict
text = "hello world"
print("Character Count:", count_characters(text))
These examples cover fundamental programming concepts typically required in the Class 12 curriculum, including conditionals, loops, functions, lists, dictionaries, file handling, and some commonly used algorithms. Let me know if you’d like more details on any specific program or topic!'''
def studyexamples():

    """
    Retrieve common Python programming examples for NCERT Class 12.

    This function returns the content of the global variable 's', which contains
    a collection of Python programming examples and codes relevant for Class 12
    students. These examples cover basic syntax, conditionals, loops, functions,
    string manipulation, lists, dictionaries, file handling, and more.
    """

    global s
    return s
k='''
For NCERT Class 12 students in India, Python programming focuses on fundamental concepts and includes many standard functions. Below is a categorized list of common Python functions and methods relevant to the Class 12 syllabus. This list includes standard library functions, mathematical operations, and string manipulation, which are typically covered at this level.

1. Basic Built-in Functions
These are core Python functions that students often use:

print() - Displays output to the console.
input() - Accepts user input.
len() - Returns the length of an iterable (e.g., string, list).
type() - Returns the type of an object.
int(), float(), str() - Type conversion functions.
round() - Rounds a floating-point number to the nearest integer.
abs() - Returns the absolute value of a number.
min() / max() - Returns the minimum or maximum of an iterable.
sum() - Sums up elements in an iterable.
sorted() - Returns a sorted list from an iterable.
range() - Generates a sequence of numbers.
enumerate() - Provides a counter with an iterable.
zip() - Combines multiple iterables element-wise.
reversed() - Reverses the order of an iterable.
2. Mathematical Functions (Requires import math)
The math module provides additional functions for mathematical calculations:

math.sqrt() - Returns the square root of a number.
math.pow() - Raises a number to a power.
math.ceil() / math.floor() - Rounds up or down to the nearest integer.
math.factorial() - Returns the factorial of an integer.
math.gcd() - Computes the greatest common divisor.
math.sin(), math.cos(), math.tan() - Trigonometric functions.
math.log() - Returns the natural logarithm of a number.
math.pi - Constant representing the value of π.
3. String Functions and Methods
Strings in Python have many useful methods for manipulation:

str.upper() / str.lower() - Converts to uppercase or lowercase.
str.capitalize() - Capitalizes the first character.
str.find() - Finds the first occurrence of a substring.
str.replace() - Replaces substrings within a string.
str.split() - Splits a string by a delimiter.
str.join() - Joins elements of an iterable with a string separator.
str.strip() - Removes leading and trailing whitespace.
str.startswith() / str.endswith() - Checks if a string starts or ends with a specific substring.
str.isalpha(), str.isdigit(), str.isalnum() - Checks if the string has alphabetic, numeric, or alphanumeric characters.
4. List Functions and Methods
Lists, a common data structure, have a variety of methods:

list.append() - Adds an element to the end of the list.
list.extend() - Adds multiple elements to the end.
list.insert() - Inserts an element at a specified index.
list.pop() - Removes and returns an element at an index.
list.remove() - Removes the first occurrence of a specified value.
list.sort() / sorted() - Sorts elements in ascending or descending order.
list.reverse() - Reverses the list in place.
list.index() - Returns the index of the first occurrence of a specified value.
list.count() - Counts occurrences of a specified value.
5. Dictionary Functions and Methods
Dictionaries are also commonly used and have unique methods:

dict.keys() - Returns all keys in the dictionary.
dict.values() - Returns all values.
dict.items() - Returns all key-value pairs as tuples.
dict.get() - Returns the value for a specified key.
dict.update() - Updates the dictionary with another dictionary’s key-value pairs.
dict.pop() - Removes and returns a value by key.
6. Additional Functions
Some additional Python functions that may be introduced:

all() - Returns True if all elements in an iterable are True.
any() - Returns True if any element in an iterable is True.
map() - Applies a function to all items in an iterable.
filter() - Filters items in an iterable based on a function.
lambda functions - Anonymous functions often used with map() and filter().
7. File Handling Functions
Basic functions to handle file operations:

open() - Opens a file.
file.read() - Reads data from a file.
file.write() - Writes data to a file.
file.readline() / file.readlines() - Reads lines from a file.
file.close() - Closes the file.
These functions cover most of what’s included in the NCERT Class 12 curriculum for Python programming. Let me know if you’d like more information on any specific function!'''
def studyfunctions():

    """
    Retrieve detailed study material.

    This function returns the content of the global variable 'k', which contains
    a categorized list of common Python functions and methods relevant to the
    Class 12 syllabus. It includes standard library functions, mathematical
    operations, string manipulation, list and dictionary methods, and file
    handling functions.
    """

    global k
    return k
m='''
Here’s a brief explanation of each topic in Python for Class 12 students, broken down by each major concept as per the NCERT syllabus. You can use this as a guide or as study material for students.

1. Basic Operations
Input and Output: In Python, input() is used to take input from the user, and print() is used to display output. For example:


name = input("Enter your name: ")
print(f"Hello, {name}")
Arithmetic Operations: Python allows basic arithmetic operations like addition (+), subtraction (-), multiplication (*), division (/), modulus (%), and exponentiation (**).


a = 5
b = 3
print(a + b)  # Output: 8
2. Conditional Statements (if, elif, else)
Conditionals: Conditional statements help us make decisions in a program. The if statement runs a block of code if the condition is True, elif is used for additional checks, and else runs if all conditions are False.

Example:


x = 10
if x > 0:
    print("Positive")
elif x == 0:
    print("Zero")
else:
    print("Negative")
3. Loops (for, while)
For Loop: Used to iterate over a sequence (like a list, string, or range). It repeats a block of code a fixed number of times.


for i in range(5):  # Will print numbers 0 to 4
    print(i)
While Loop: Executes a block of code as long as the given condition is True.


i = 0
while i < 5:
    print(i)
    i += 1
4. Functions
Functions: Functions are blocks of reusable code that perform specific tasks. You define a function with the def keyword, and it can take inputs (parameters) and return outputs.

Example:


def greet(name):
    return f"Hello, {name}"

print(greet("Alice"))  # Output: Hello, Alice
Functions help in breaking down complex problems into smaller, manageable tasks.

5. String Manipulation
Strings: Strings are sequences of characters. You can perform various operations like concatenation, slicing, and case conversion.


text = "Hello World"
print(text.upper())  # Output: "HELLO WORLD"
print(text[0:5])     # Output: "Hello"
String Methods:

upper(): Converts to uppercase.
lower(): Converts to lowercase.
replace(): Replaces a substring.
Example:


greeting = "Hello"
greeting = greeting.replace("e", "a")  # Output: "Hallo"
6. Lists
Lists: Lists are ordered collections that can store multiple items of different types. You can add, remove, and access elements in a list.

Example:


fruits = ["apple", "banana", "cherry"]
fruits.append("orange")  # Add to the end
fruits.remove("banana")  # Remove item
print(fruits)  # Output: ['apple', 'cherry', 'orange']
7. Dictionaries
Dictionaries: A dictionary is a collection of key-value pairs. Each key is unique and is used to access the corresponding value.

Example:


student = {"name": "John", "age": 18, "marks": 90}
print(student["name"])  # Output: John
student["age"] = 19  # Update value
8. File Handling
File Handling: Python can read from and write to files using the open() function. You can open a file in different modes (r for read, w for write, etc.).

Example:


# Writing to a file
with open("example.txt", "w") as file:
    file.write("Hello, World!")

# Reading from a file
with open("example.txt", "r") as file:
    content = file.read()
    print(content)  # Output: Hello, World!
9. Algorithms (Prime, Armstrong, etc.)
Prime Numbers: A prime number is a number that is greater than 1 and only divisible by 1 and itself.

Example of checking if a number is prime:


def is_prime(num):
    if num < 2:
        return False
    for i in range(2, num):
        if num % i == 0:
            return False
    return True
Armstrong Numbers: An Armstrong number is a number where the sum of its digits raised to the power of the number of digits is equal to the number itself.

Example:


def is_armstrong(num):
    digits = [int(digit) for digit in str(num)]
    power = len(digits)
    total = sum(d ** power for d in digits)
    return total == num
10. SQL (Structured Query Language)
SQL is used to interact with relational databases. It allows you to insert, update, delete, and query data in a database.

Database Connection: To interact with databases in Python, the sqlite3 module is commonly used. Here’s an example of how to connect and run SQL queries.

import sqlite3

# Connecting to SQLite database
conn = sqlite3.connect('students.db')
cursor = conn.cursor()

# Creating a table
cursor.execute(\'\'\'CREATE TABLE IF NOT EXISTS students (id INTEGER PRIMARY KEY, name TEXT, age INTEGER)\'\'\')

# Inserting data
cursor.execute("INSERT INTO students (name, age) VALUES ('Alice', 18)")

# Querying data
cursor.execute("SELECT * FROM students")
print(cursor.fetchall())  # Fetches all rows

conn.commit()
conn.close()
11. Stack (LIFO - Last In, First Out)
A stack is a linear data structure where elements are added or removed from the same end. It follows the LIFO principle: the last element added is the first one to be removed.

Example of implementing a stack:


class Stack:
    def __init__(self):
        self.stack = []

    def push(self, item):
        """Pushes an item onto the stack."""
        self.stack.append(item)

    def pop(self):
        """Pops an item from the stack."""
        if not self.is_empty():
            return self.stack.pop()
        else:
            return "Stack is empty"

    def is_empty(self):
        """Checks if the stack is empty."""
        return len(self.stack) == 0

    def peek(self):
        """Returns the top element without removing it."""
        if not self.is_empty():
            return self.stack[-1]
        else:
            return "Stack is empty"
12. Queue (FIFO - First In, First Out)
A queue is a linear data structure where elements are added at the rear and removed from the front. It follows the FIFO principle: the first element added is the first one to be removed.

Example of implementing a queue:


class Queue:
    def __init__(self):
        self.queue = []

    def enqueue(self, item):
        """Adds an item to the queue."""
        self.queue.append(item)

    def dequeue(self):
        """Removes an item from the queue."""
        if not self.is_empty():
            return self.queue.pop(0)
        else:
            return "Queue is empty"

    def is_empty(self):
        """Checks if the queue is empty."""
        return len(self.queue) == 0

    def front(self):
        """Returns the front element without removing it."""
        if not self.is_empty():
            return self.queue[0]
        else:
            return "Queue is empty"
13. CSV (Comma-Separated Values)
CSV is a file format used to store tabular data. Python provides the csv module to read from and write to CSV files.

Example of reading and writing CSV files:


import csv

# Writing to a CSV file
with open('data.csv', mode='w', newline='') as file:
    writer = csv.writer(file)
    writer.writerow(['Name', 'Age', 'Country'])
    writer.writerow(['Alice', 18, 'USA'])
    writer.writerow(['Bob', 20, 'UK'])

# Reading from a CSV file
with open('data.csv', mode='r') as file:
    reader = csv.reader(file)
    for row in reader:
        print(row)
14. Binary Files
Binary files are files that contain data in binary format (0s and 1s). You can read and write binary files using Python's built-in open() function with the 'b' mode.

Example of reading and writing binary files:


# Writing to a binary file
with open('example.bin', 'wb') as file:
    data = b'\\x00\\x01\\x02\\x03\\x04'
    file.write(data)

# Reading from a binary file
with open('example.bin', 'rb') as file:
    content = file.read()
    print(content)  # Output: b'\\x00\\x01\\x02\\x03\\x04'
15. Exceptions and Error Handling
Exceptions are runtime errors that occur in Python. You can handle these errors using try, except, else, and finally blocks to prevent the program from crashing.

Example of error handling:


def divide(a, b):
    try:
        result = a / b
    except ZeroDivisionError:
        return "Error: Division by zero"
    except TypeError:
        return "Error: Invalid input types"
    else:
        return result
    finally:
        print("Execution complete.")

print(divide(10, 0))  # Output: Error: Division by zero
Common Python Errors:

ZeroDivisionError: Raised when dividing by zero.
TypeError: Raised when an operation is applied to an object of inappropriate type.
IndexError: Raised when an invalid index is accessed in a list.
ValueError: Raised when a function receives an argument of the correct type but inappropriate value.
16. Recursion
Recursion is when a function calls itself to solve smaller instances of the same problem. It is important to define a base case to avoid infinite recursion.

Example of calculating factorial using recursion:


def factorial(n):
    if n == 0:
        return 1
    else:
        return n * factorial(n - 1)

print(factorial(5))  # Output: 120
17. Sets
Sets are unordered collections of unique elements. They are useful for performing operations like union, intersection, and difference.

Example of set operations:


set1 = {1, 2, 3, 4}
set2 = {3, 4, 5, 6}

# Union
print(set1 | set2)  # Output: {1, 2, 3, 4, 5, 6}

# Intersection
print(set1 & set2)  # Output: {3, 4}

# Difference
print(set1 - set2)  # Output: {1, 2}
18. Tuples
Tuples are similar to lists, but they are immutable. Once created, you cannot modify their values.

Example of tuple operations:


tuple1 = (1, 2, 3, 4)
print(tuple1[0])  # Output: 1
19. Lambda Functions
Lambda functions are small anonymous functions defined with the lambda keyword. They are typically used for short, throwaway functions.

Example of a lambda function:


square = lambda x: x * x
print(square(5))  # Output: 25
20. List Comprehensions
List Comprehensions provide a concise way to create lists. It allows you to transform one list into another by applying an expression.

Example of list comprehension:


numbers = [1, 2, 3, 4, 5]
squares = [x**2 for x in numbers]  # Output: [1, 4, 9, 16, 25]
Conclusion
With these additional topics, Class 12 students will have a broad understanding of Python and related concepts. By learning these, students will be equipped to handle a wide variety of problems, from database operations with SQL to advanced data structures like stacks, queues, and recursion.
These topics will also provide a strong foundation for future studies in computer science, data science, and software development.'''
def studyshort():

    """
    Retrieve short study notes.

    This function returns the content of the global variable 'm', which contains
    brief explanations and examples on Python topics intended for Class 12 students.
    These notes cover basic operations, conditionals, loops, functions, and more,
    providing a concise overview of essential Python concepts.
    """

    global m
    return m
z='''
To work with MySQL databases in Python, you can use the mysql-connector-python library. This allows you to interact with MySQL databases, execute queries, and handle data. Here's how you can install and use MySQL Connector for Python.

1. Installation
First, you need to install the mysql-connector-python package. You can install it using pip:

bash
Copy code
pip install mysql-connector-python
2. Connecting to a MySQL Database
To connect to a MySQL database, you need the hostname, username, password, and database name.

Here’s how to connect to the MySQL database:


import mysql.connector

# Establishing the connection
conn = mysql.connector.connect(
    host="localhost",       # MySQL server hostname or IP
    user="your_username",   # MySQL username
    password="your_password",  # MySQL password
    database="your_database"   # Database name
)

# Creating a cursor object
cursor = conn.cursor()

# Checking the connection
if conn.is_connected():
    print("Successfully connected to MySQL database!")

# Don't forget to close the connection
conn.close()
3. Creating a Database and Table
Once connected, you can create a database and a table for storing your data.

Example to create a database and a table:


# Establish connection
conn = mysql.connector.connect(
    host="localhost",
    user="your_username",
    password="your_password"
)

cursor = conn.cursor()

# Creating a database
cursor.execute("CREATE DATABASE IF NOT EXISTS studentDB")

# Selecting the database
conn.database = 'studentDB'

# Creating a table
cursor.execute("""
CREATE TABLE IF NOT EXISTS students (
    id INT AUTO_INCREMENT PRIMARY KEY,
    name VARCHAR(255) NOT NULL,
    age INT NOT NULL,
    grade VARCHAR(10)
)
""")
print("Database and Table Created Successfully!")

conn.close()
4. Inserting Data into a Table
You can insert data into your MySQL database using the INSERT INTO SQL statement.

Example to insert data into the students table:


# Connecting to the database
conn = mysql.connector.connect(
    host="localhost",
    user="your_username",
    password="your_password",
    database="studentDB"
)

cursor = conn.cursor()

# Inserting data into the students table
cursor.execute("""
INSERT INTO students (name, age, grade)
VALUES (%s, %s, %s)
""", ("Alice", 18, "A"))

# Commit the transaction
conn.commit()

print("Data inserted successfully!")

# Close the connection
conn.close()
5. Retrieving Data from a Table
To retrieve data, you use the SELECT SQL statement.

Example to fetch all records from the students table:


# Connecting to the database
conn = mysql.connector.connect(
    host="localhost",
    user="your_username",
    password="your_password",
    database="studentDB"
)

cursor = conn.cursor()

# Fetching data from the students table
cursor.execute("SELECT * FROM students")

# Fetching all results
result = cursor.fetchall()

for row in result:
    print(row)

# Close the connection
conn.close()
6. Updating Data in a Table
To update existing data, you can use the UPDATE SQL statement.

Example to update the grade of a student:


# Connecting to the database
conn = mysql.connector.connect(
    host="localhost",
    user="your_username",
    password="your_password",
    database="studentDB"
)

cursor = conn.cursor()

# Updating data in the students table
cursor.execute("""
UPDATE students
SET grade = %s
WHERE name = %s
""", ("B", "Alice"))

# Commit the transaction
conn.commit()

print("Data updated successfully!")

# Close the connection
conn.close()
7. Deleting Data from a Table
To delete data, you can use the DELETE SQL statement.

Example to delete a student record:


# Connecting to the database
conn = mysql.connector.connect(
    host="localhost",
    user="your_username",
    password="your_password",
    database="studentDB"
)

cursor = conn.cursor()

# Deleting data from the students table
cursor.execute("DELETE FROM students WHERE name = %s", ("Alice",))

# Commit the transaction
conn.commit()

print("Data deleted successfully!")

# Close the connection
conn.close()
8. Handling Exceptions
When working with databases, it's important to handle exceptions that may occur during database operations.

Example of exception handling in MySQL queries:

python
Copy code
try:
    conn = mysql.connector.connect(
        host="localhost",
        user="your_username",
        password="your_password",
        database="studentDB"
    )

    cursor = conn.cursor()

    # Example query that could raise an exception
    cursor.execute("SELECT * FROM non_existing_table")

except mysql.connector.Error as err:
    print(f"Error: {err}")
finally:
    if conn.is_connected():
        conn.close()
9. Closing the Connection
Remember to always close the connection after you're done with your database operations to free up resources.


if conn.is_connected():
    conn.close()
    print("Connection closed")
Conclusion
This brief overview shows how to connect to a MySQL database using mysql-connector-python, perform CRUD (Create, Read, Update, Delete) operations, and handle basic errors. This will be helpful for Class 12 students to integrate database operations into their Python programs.

You can combine these operations with other topics, such as SQL queries, data analysis, and user interaction, to build a complete project or solve real-world problems using Python.'''
def studymysqlpython():

    """
    Retrieve MySQL study notes for Python.

    This function accesses the global variable 'z', which contains comprehensive
    notes and examples on using MySQL with Python. These notes include instructions
    on how to connect to a database, perform CRUD operations, and handle exceptions.
    
    Returns
    -------
    str
        The content of the global variable 'z', providing a detailed guide for 
        Class 12 students on integrating MySQL database operations into Python programs.
    """

    global z
    return z
a='''
1. Python Basics
Data Types
Python supports several built-in data types:

int: Integer numbers (e.g., 5, -10)
float: Decimal numbers (e.g., 3.14, -0.001)
str: Strings of characters (e.g., "hello", 'Python')
bool: Boolean values (True, False)
list: A collection of ordered elements (e.g., [1, 2, 3])
tuple: An immutable collection (e.g., (1, 2, 3))
dict: A collection of key-value pairs (e.g., {"name": "John", "age": 18})
set: A collection of unique unordered elements (e.g., {1, 2, 3})
NoneType: A special type representing None (used for null values)
Variables and Constants
Variables hold data of any data type and can be reassigned.
Constants are typically written in uppercase letters, but Python does not have a built-in constant data type.
2. Control Flow
Conditional Statements
if: Executes a block of code if the condition is True.
elif: Checks additional conditions if the if condition is False.
else: Executes when all the previous conditions are False.
Example:


x = 10
if x > 0:
    print("Positive")
elif x == 0:
    print("Zero")
else:
    print("Negative")
Loops
for loop: Used to iterate over a sequence (like lists, ranges).
while loop: Repeats a block of code as long as the condition is True.
Example:


# For loop example
for i in range(1, 6):  # From 1 to 5
    print(i)

# While loop example
x = 0
while x < 5:
    print(x)
    x += 1
Break and Continue
break: Exits the loop prematurely.
continue: Skips the current iteration and moves to the next one.
3. Functions
Defining Functions
Functions help you organize your code into smaller, reusable parts. Functions are defined using the def keyword.

Example:


def greet(name):
    return f"Hello, {name}!"

print(greet("Alice"))  # Output: Hello, Alice!
Arguments and Return Values
You can pass parameters to functions and return values using the return keyword.
default arguments: You can set default values for parameters.
variable-length arguments: Use *args and **kwargs for a variable number of arguments.
Example with default argument:


def greet(name="Student"):
    return f"Hello, {name}!"

print(greet())          # Output: Hello, Student!
print(greet("Alice"))   # Output: Hello, Alice!
4. Data Structures
Lists
Lists are mutable, ordered collections that can store mixed data types.
Operations on lists:

Append: Adds an item to the end.
Remove: Removes an item by value.
Pop: Removes an item by index and returns it.
Example:


fruits = ["apple", "banana", "cherry"]
fruits.append("orange")  # Adds "orange" to the list
fruits.remove("banana")  # Removes "banana"
print(fruits)  # Output: ['apple', 'cherry', 'orange']
Tuples
Tuples are similar to lists but are immutable (cannot be changed after creation).
Example:


colors = ("red", "green", "blue")
# colors[0] = "yellow"  # This will raise an error since tuples are immutable
Dictionaries
Dictionaries store key-value pairs. Keys are unique, and values can be of any data type.
Example:


student = {"name": "John", "age": 18, "marks": 90}
print(student["name"])  # Output: John
student["age"] = 19  # Update age
Sets
Sets are unordered collections with unique elements.
Example:


unique_numbers = {1, 2, 3, 3}
print(unique_numbers)  # Output: {1, 2, 3}
5. File Handling
Python provides built-in methods to work with files, allowing you to read from and write to files.

Reading from a File

with open("file.txt", "r") as file:
    content = file.read()
    print(content)
Writing to a File

with open("file.txt", "w") as file:
    file.write("Hello, world!")
Handling CSV Files
Use the csv module to read and write CSV files.


import csv

# Writing to CSV
with open("data.csv", mode="w", newline="") as file:
    writer = csv.writer(file)
    writer.writerow(["Name", "Age", "Country"])
    writer.writerow(["Alice", 18, "USA"])

# Reading from CSV
with open("data.csv", mode="r") as file:
    reader = csv.reader(file)
    for row in reader:
        print(row)
6. MySQL with Python
Connecting to MySQL Database

import mysql.connector

conn = mysql.connector.connect(
    host="localhost",
    user="root",
    password="yourpassword",
    database="yourdatabase"
)

cursor = conn.cursor()
Executing SQL Queries
Example of creating a table:


cursor.execute("""
CREATE TABLE IF NOT EXISTS students (
    id INT AUTO_INCREMENT PRIMARY KEY,
    name VARCHAR(255),
    age INT,
    grade VARCHAR(10)
)
""")
CRUD Operations in MySQL
Insert Data:

cursor.execute("""
INSERT INTO students (name, age, grade)
VALUES (%s, %s, %s)
""", ("Alice", 18, "A"))
conn.commit()
Select Data:

cursor.execute("SELECT * FROM students")
for row in cursor.fetchall():
    print(row)
Update Data:

cursor.execute("""
UPDATE students
SET grade = %s
WHERE name = %s
""", ("B", "Alice"))
conn.commit()
Delete Data:

cursor.execute("DELETE FROM students WHERE name = %s", ("Alice",))
conn.commit()
7. Exceptions and Error Handling
Handling Exceptions
Use try, except, else, and finally blocks to handle exceptions gracefully.


try:
    x = 10 / 0
except ZeroDivisionError:
    print("Cannot divide by zero!")
else:
    print("No error occurred")
finally:
    print("Execution finished.")
8. Recursion
Recursion is when a function calls itself to solve smaller instances of the same problem. It’s important to have a base case to prevent infinite recursion.

Example of calculating factorial recursively:


def factorial(n):
    if n == 0:
        return 1
    else:
        return n * factorial(n - 1)

print(factorial(5))  # Output: 120
9. Advanced Topics
Lambda Functions
Lambda functions are small anonymous functions defined using the lambda keyword. They are typically used for short, one-line functions.

Example:


square = lambda x: x ** 2
print(square(5))  # Output: 25
List Comprehensions
List comprehensions provide a concise way to create lists from other iterables.

Example:


numbers = [1, 2, 3, 4, 5]
squares = [x**2 for x in numbers]
print(squares)  # Output: [1, 4, 9, 16, 25]
Decorators
Decorators are a way to modify the behavior of a function or class. They are often used for logging, access control, caching, etc.

Example:


def decorator(func):
    def wrapper():
        print("Before function call")
        func()
        print("After function call")
    return wrapper

@decorator
def say_hello():
    print("Hello!")

say_hello()
Conclusion
These notes cover a wide range of topics in Python, from basics like variables, data types, and functions, to more advanced topics like file handling, MySQL, recursion, and decorators. This collection provides a strong foundation for students to tackle both theoretical and practical problems in Python programming.

By practicing these concepts through examples, students can build confidence and gain a deeper understanding of Python's capabilities. The application of these concepts in real-world scenarios such as working with databases, file handling, and data manipulation is key to mastering Python.
'''
def notes():

    """
    Returns the value of the global variable 'a'.

    This function retrieves and returns the content of the global variable 'a',
    which contains a comprehensive collection of Python programming notes. These
    notes cover a variety of topics in Python, providing a solid foundation for
    students to understand and apply Python concepts.
    """

    global a
    return a
x='''
1. Python Basics
Variables and Data Types
Python allows you to assign values to variables, and it automatically determines the data type based on the value assigned. Below are some examples of basic data types:


# Integers
x = 5
print(x)  # Output: 5

# Floats
y = 3.14
print(y)  # Output: 3.14

# Strings
name = "Alice"
print(name)  # Output: Alice

# Boolean
is_active = True
print(is_active)  # Output: True

# Lists
fruits = ["apple", "banana", "cherry"]
print(fruits)  # Output: ['apple', 'banana', 'cherry']

# Tuple
coordinates = (4, 5)
print(coordinates)  # Output: (4, 5)

# Dictionary
person = {"name": "John", "age": 30}
print(person)  # Output: {'name': 'John', 'age': 30}

# Set
numbers = {1, 2, 3}
print(numbers)  # Output: {1, 2, 3}
2. Conditional Statements
If, Elif, Else
Conditional statements are used to make decisions in your code based on certain conditions.


x = 15

if x > 10:
    print("x is greater than 10")  # Output: x is greater than 10
elif x == 10:
    print("x is equal to 10")
else:
    print("x is less than 10")
3. Loops
For Loop
The for loop is used to iterate over a sequence (like a list, tuple, string, etc.).


fruits = ["apple", "banana", "cherry"]

# Iterating over a list with a for loop
for fruit in fruits:
    print(fruit)
Output:


apple
banana
cherry
While Loop
The while loop executes as long as the condition is True.


x = 1
while x <= 5:
    print(x)
    x += 1
Output:


1
2
3
4
5
4. Functions
Defining Functions
Functions help break down the code into reusable blocks.


def greet(name):
    return f"Hello, {name}!"

print(greet("Alice"))  # Output: Hello, Alice!
Return and Arguments
Functions can take arguments and return values.


def add(a, b):
    return a + b

print(add(3, 5))  # Output: 8
5. Data Structures
Lists
Lists are ordered collections that can store a variety of data types.


# List with mixed data types
my_list = [1, 3.14, "hello", True]
print(my_list)  # Output: [1, 3.14, 'hello', True]

# List operations
my_list.append("new item")
print(my_list)  # Output: [1, 3.14, 'hello', True, 'new item']

my_list.remove(3.14)
print(my_list)  # Output: [1, 'hello', True, 'new item']
Tuples
Tuples are immutable, which means once defined, their values cannot be changed.


my_tuple = (1, 2, 3)
print(my_tuple)  # Output: (1, 2, 3)

# Accessing an element
print(my_tuple[1])  # Output: 2
Dictionaries
Dictionaries store data in key-value pairs.


person = {"name": "Alice", "age": 25, "city": "New York"}
print(person)  # Output: {'name': 'Alice', 'age': 25, 'city': 'New York'}

# Accessing values
print(person["name"])  # Output: Alice

# Adding new key-value pair
person["job"] = "Engineer"
print(person)  # Output: {'name': 'Alice', 'age': 25, 'city': 'New York', 'job': 'Engineer'}
Sets
Sets store unique values and are unordered.


my_set = {1, 2, 3, 4}
print(my_set)  # Output: {1, 2, 3, 4}

# Adding to set
my_set.add(5)
print(my_set)  # Output: {1, 2, 3, 4, 5}
6. File Handling
Reading Files
To read a file, we use the open() function.


# Read from a file (Assuming a file "example.txt" exists)
with open("example.txt", "r") as file:
    content = file.read()
    print(content)
Writing to Files
You can write to a file using the write() method.


with open("example.txt", "w") as file:
    file.write("This is a test.")
Reading and Writing CSV Files
CSV (Comma Separated Values) files can be handled using the csv module.


import csv

# Writing to CSV
with open("students.csv", "w", newline="") as file:
    writer = csv.writer(file)
    writer.writerow(["Name", "Age", "Grade"])
    writer.writerow(["Alice", 18, "A"])
    writer.writerow(["Bob", 17, "B"])

# Reading from CSV
with open("students.csv", "r") as file:
    reader = csv.reader(file)
    for row in reader:
        print(row)
Output:


['Name', 'Age', 'Grade']
['Alice', '18', 'A']
['Bob', '17', 'B']
7. Exception Handling
Try and Except Block
You can handle exceptions using try and except blocks.


try:
    x = 10 / 0  # This will raise a ZeroDivisionError
except ZeroDivisionError:
    print("Cannot divide by zero!")
Output:


Cannot divide by zero!
Finally Block
The finally block is always executed, even if there is an exception.


try:
    x = 10 / 2
except ZeroDivisionError:
    print("Error!")
finally:
    print("This is executed no matter what!")
Output:


This is executed no matter what!
8. MySQL with Python
Connecting to MySQL Database
Here is how you can connect to a MySQL database using mysql-connector-python.


import mysql.connector

# Connecting to the database
conn = mysql.connector.connect(
    host="localhost",
    user="your_username",
    password="your_password",
    database="your_database"
)

cursor = conn.cursor()

# Creating a table
cursor.execute("""
CREATE TABLE IF NOT EXISTS students (
    id INT AUTO_INCREMENT PRIMARY KEY,
    name VARCHAR(255),
    age INT,
    grade VARCHAR(10)
)
""")

conn.commit()
CRUD Operations in MySQL
Insert Data:

cursor.execute("""
INSERT INTO students (name, age, grade)
VALUES (%s, %s, %s)
""", ("John", 18, "A"))
conn.commit()

print("Data inserted successfully!")
Read Data:

cursor.execute("SELECT * FROM students")
for row in cursor.fetchall():
    print(row)
Output:


(1, 'John', 18, 'A')
Update Data:

cursor.execute("""
UPDATE students
SET grade = %s
WHERE name = %s
""", ("B", "John"))
conn.commit()

print("Data updated successfully!")
Delete Data:

cursor.execute("DELETE FROM students WHERE name = %s", ("John",))
conn.commit()

print("Data deleted successfully!")
9. Recursion
Factorial Calculation
Recursion involves a function calling itself. Below is an example of calculating the factorial of a number using recursion.


def factorial(n):
    if n == 0:
        return 1
    else:
        return n * factorial(n - 1)

print(factorial(5))  # Output: 120
10. Lambda Functions
Lambda functions are small anonymous functions that can have any number of arguments but only one expression.


# Lambda function to find square of a number
square = lambda x: x ** 2
print(square(4))  # Output: 16
11. List Comprehensions
List comprehensions provide a concise way to create lists.


numbers = [1, 2, 3, 4, 5]
squares = [x ** 2 for x in numbers]
print(squares)  # Output: [1, 4, 9, 16, 25]
Conclusion
These expanded notes and examples should give students a more detailed understanding of Python programming. Each topic is explained with practical examples that provide immediate feedback through outputs, helping students practice and learn. This approach enhances the learning experience and prepares students for exams, projects, or real-world programming scenarios.'''
def basics():

    """
    Basic notes for study
    
    This function does not take any arguments and does not do anything with the global variable x.
    It is only used to group the basic notes for study into a single function.
    """
    

    global x
    return x

df='''
Here's a concise guide covering MySQL, its core concepts, and essential commands. This should be helpful for understanding SQL basics and common operations!

MySQL Overview
MySQL is an open-source relational database management system (RDBMS) based on Structured Query Language (SQL). It’s used to manage databases and is known for its speed, reliability, and ease of use.

Basic Concepts
Database: A collection of tables that organize data in a structured way.
Table: Stores data in rows and columns.
Row: Represents a single record in a table.
Column: Represents an attribute or field in a table.
Primary Key: A unique identifier for each row in a table.
Foreign Key: A key used to link two tables together.
Basic MySQL Commands
1. Database Commands
Create a Database

CREATE DATABASE database_name;
Use a Database

USE database_name;
Show Databases

SHOW DATABASES;
Drop a Database

DROP DATABASE database_name;
2. Table Commands
Create a Table

CREATE TABLE table_name (
    column1 datatype constraints,
    column2 datatype constraints,
    ...
);
Example:
sql
Copy code
CREATE TABLE students (
    id INT PRIMARY KEY,
    name VARCHAR(100),
    age INT
);
Show Tables

SHOW TABLES;
Describe Table Structure

DESCRIBE table_name;
Drop a Table

DROP TABLE table_name;
3. Inserting Data
Insert Data into a Table

INSERT INTO table_name (column1, column2, ...) VALUES (value1, value2, ...);
Example:

INSERT INTO students (id, name, age) VALUES (1, 'Alice', 20);
4. Retrieving Data (SELECT Command)
Select All Data

SELECT * FROM table_name;
Select Specific Columns

SELECT column1, column2 FROM table_name;
Using WHERE Clause (Conditional Query)

SELECT * FROM table_name WHERE condition;
Example:

SELECT * FROM students WHERE age > 18;
Using ORDER BY (Sorting Data)

SELECT * FROM table_name ORDER BY column_name ASC|DESC;
Using LIMIT (Restricting Number of Rows Returned)

SELECT * FROM table_name LIMIT number;
5. Updating Data
Update Data in a Table

UPDATE table_name SET column1 = value1, column2 = value2 WHERE condition;
Example:

UPDATE students SET age = 21 WHERE name = 'Alice';
6. Deleting Data
Delete Data from a Table

DELETE FROM table_name WHERE condition;
Example:

DELETE FROM students WHERE age < 18;
7. Constraints
Primary Key: Ensures that each record has a unique value for this field.

CREATE TABLE table_name (
    column_name datatype PRIMARY KEY
);
Foreign Key: Links two tables together.

CREATE TABLE orders (
    order_id INT,
    student_id INT,
    FOREIGN KEY (student_id) REFERENCES students(id)
);
Not Null: Ensures that a column cannot have a NULL value.

CREATE TABLE table_name (
    column_name datatype NOT NULL
);
8. Aggregate Functions
COUNT(): Counts the number of rows in a table.

SELECT COUNT(*) FROM table_name;
SUM(): Calculates the sum of a numeric column.

SELECT SUM(column_name) FROM table_name;
AVG(): Calculates the average value of a numeric column.

SELECT AVG(column_name) FROM table_name;
MIN() and MAX(): Finds the minimum and maximum values.

SELECT MIN(column_name) FROM table_name;
SELECT MAX(column_name) FROM table_name;
9. Grouping Data
GROUP BY: Groups rows that have the same values in specified columns into summary rows.

SELECT column_name, COUNT(*) FROM table_name GROUP BY column_name;
10. Joins
Joins are used to combine rows from two or more tables based on a related column.

INNER JOIN: Returns rows with matching values in both tables.

SELECT a.column_name, b.column_name
FROM table1 a
INNER JOIN table2 b ON a.common_field = b.common_field;
LEFT JOIN (or LEFT OUTER JOIN): Returns all rows from the left table, even if there’s no match in the right table.

SELECT a.column_name, b.column_name
FROM table1 a
LEFT JOIN table2 b ON a.common_field = b.common_field;
RIGHT JOIN: Returns all rows from the right table, even if there’s no match in the left table.

SELECT a.column_name, b.column_name
FROM table1 a
RIGHT JOIN table2 b ON a.common_field = b.common_field;
Common Data Types in MySQL
INT: Integer numbers (whole numbers).
FLOAT/DOUBLE: Floating-point numbers.
CHAR(size): Fixed-length character strings.
VARCHAR(size): Variable-length character strings.
DATE: Date in YYYY-MM-DD format.
TIME: Time in HH:MM:SS format.
DATETIME: Date and time in YYYY-MM-DD HH:MM:SS format.
BOOLEAN: True or False values (stored as 1 or 0).
SQL Clauses and Keywords
DISTINCT: Used to select unique values.

SELECT DISTINCT column_name FROM table_name;
IN: Checks if a value is within a set of values.
sql
Copy code
SELECT * FROM table_name WHERE column_name IN (value1, value2, ...);
BETWEEN: Checks if a value falls within a range.
sql
Copy code
SELECT * FROM table_name WHERE column_name BETWEEN value1 AND value2;
LIKE: Used for pattern matching.
sql
Copy code
SELECT * FROM table_name WHERE column_name LIKE 'pattern';
IS NULL: Checks for NULL values.

SELECT * FROM table_name WHERE column_name IS NULL;
Example of a Simple SQL Database Operation
Assume we have two tables: students and courses.

Create Tables


CREATE TABLE students (
    id INT PRIMARY KEY,
    name VARCHAR(50),
    age INT
);

CREATE TABLE courses (
    course_id INT PRIMARY KEY,
    course_name VARCHAR(100),
    student_id INT,
    FOREIGN KEY (student_id) REFERENCES students(id)
);
Insert Data


INSERT INTO students (id, name, age) VALUES (1, 'Alice', 20);
INSERT INTO students (id, name, age) VALUES (2, 'Bob', 22);

INSERT INTO courses (course_id, course_name, student_id) VALUES (101, 'Math', 1);
INSERT INTO courses (course_id, course_name, student_id) VALUES (102, 'Science', 2);
Retrieve Data with Join


SELECT students.name, courses.course_name
FROM students
INNER JOIN courses ON students.id = courses.student_id;
Output:


name   | course_name
--------------------
Alice  | Math
Bob    | Science
This guide should provide a strong foundation for understanding MySQL and SQL commands, and help you confidently execute basic and intermediate database operations!'''
def mysqlnotes():

    """
    Returns an overview guide about MySQL, covering core concepts and essential commands.
    The guide is stored in a global variable 'df'.
    """

    global df
    return df




from setuptools import setup, find_packages

setup(
    name="ncert_learn",                   # Replace with your package name
    version="4.5.10",                     # Initial version
    author="Muhammed Shafin P",
    author_email="hejhdiss@example.com",
    description="A Python Module for NCERT Class 12 Computer Science - Learn Python and MySQL with ease!",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    url="https://github.com/hejhdiss/ncert_learn",  # Add your GitHub repository URL here
    project_urls={
        "Bug Tracker": "https://github.com/hejhdiss/ncert_learn/issues",
        "Documentation": "https://hejhdiss.github.io/ncert_learn-website/",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[
        'mysql-connector-python',
        'pytest',
        'tox',
        'requests',
        'psutil',
        'beautifulsoup4',
        'googlesearch-python',
        'python-nmap',
        'PyPDF2',
        
        
        # Add this line to include the MySQL connector
    ],
    include_package_data=True,
)



import os
current_binary_path = ""
def isbinaryfile(file_path):
    """
    Check if a file is a binary file.
    
    Parameters:
    file_path (str): Path to the file.

    Returns:
    bool: True if the file is a binary file, False otherwise.
    """
    try:
        with open(file_path, 'rb') as file:
            file.read(1024)  # Read the first 1024 bytes as a test
        return True
    except (UnicodeDecodeError, IOError):
        return False


def copybinaryfromonetoanother(src, dest):
    """
    Copies the binary content from the file given by path src to the file given by path dest.

    Parameters
    ----------
    src : str
        The path of the file to copy from.
    dest : str
        The path of the file to copy to.

    Returns
    -------
    bool
        True if the copy operation is successful, False otherwise.
    """
    if not isinstance(src, str) or not isinstance(dest, str) or not src or not dest:
        return False
    elif not isbinaryfile(src):
        return False
    try:
        with open(src, 'rb') as file_src:
            with open(dest, 'wb') as file_dest:
                file_dest.write(file_src.read())
        return True
    except (FileNotFoundError, IOError):
        return False


def openbinaryfile(file_path):
    """
    Opens the binary file given by path file_path to set it as the current working file.

    Parameters
    ----------
    file_path : str
        The path of the file to open.

    Returns
    -------
    bool
        True if the file is opened successfully, False otherwise.
    """
    global current_binary_path

    if not isinstance(file_path, str) or not file_path or not isbinaryfile(file_path):
        return False
    try:
        with open(file_path, 'rb') as file:
            current_binary_path = file_path
        return True
    except (FileNotFoundError, IOError):
        return False


def addtextobinaryfile(byte_data):
    """
    Appends a sequence of bytes to the binary file opened by openbinaryfile() function.

    Parameters
    ----------
    byte_data : bytes
        A sequence of bytes to append.

    Returns
    -------
    bool
        True if the bytes are appended successfully, False otherwise.
    """
    global current_binary_path
    if not current_binary_path or not isinstance(byte_data, bytes):
        return False
    try:
        with open(current_binary_path, 'ab') as file:
            file.write(byte_data)
        return True
    except (FileNotFoundError, IOError):
        return False


def readbinaryfile():
    """
    Reads the contents of the binary file opened by openbinaryfile() function.

    Returns
    -------
    bytes
        The contents of the file as bytes.
    """
    global current_binary_path
    try:
        with open(current_binary_path, 'rb') as file:
            return file.read()
    except (FileNotFoundError, IOError):
        return b''


def clearbinaryfile():
    """
    Clears the contents of the binary file opened by openbinaryfile() function.
    """
    global current_binary_path
    if current_binary_path:
        with open(current_binary_path, 'wb') as file:
            pass  # Overwrite with an empty file


def readbinaryspecificchunk(offset, size):
    """
    Reads a specific chunk from the binary file opened by openbinaryfile() function.

    Parameters
    ----------
    offset : int
        The starting position to read from the file.
    size : int
        The number of bytes to read.

    Returns
    -------
    bytes
        The content of the specified chunk as bytes if successful.
    bool
        False if the file is not found or any exception occurs.
    """
    global current_binary_path

    try:
        with open(current_binary_path, 'rb') as file:
            file.seek(offset)
            return file.read(size)
    except (FileNotFoundError, IOError):
        return False


# Initialize current binary path variable
def overwritebinaryfile(new_data):
    """
    Overwrites the contents of the binary file opened by openbinaryfile() with new data.

    Parameters
    ----------
    new_data : bytes
        The new content to write to the binary file.

    Returns
    -------
    bool
        True if the file is overwritten successfully, False otherwise.
    """
    global current_binary_path

    if not current_binary_path or not isinstance(new_data, bytes):
        return False
    try:
        with open(current_binary_path, 'wb') as file:
            file.write(new_data)
        return True
    except (FileNotFoundError, IOError):
        return False
import os

def binary_file_operations_advanced_mode(folder_path, operation, file_name=None, new_name=None, data=None):
    """
    Perform various binary file operations in a specified folder.

    Parameters
    ----------
    folder_path : str
        Path to the folder where the operations will be performed.
    operation : str
        The operation to perform. Supported operations:
        'create', 'write', 'append', 'read', 'clear', 'delete', 'rename', 'list_files'.
    file_name : str, optional
        The file name for the operation.
    new_name : str, optional
        New name for the file (used in 'rename').
    data : bytes, optional
        Binary data for 'write' or 'append' operations.

    Returns
    -------
    bool or bytes or list
        - For 'read': Returns file content (bytes) or False on error.
        - For 'list_files': Returns a list of files in the folder or False on error.
        - For other operations: Returns True on success, False on failure.
    """
    # Check if the folder path is valid
    if not folder_path or not os.path.isdir(folder_path):
        return False

    try:
        file_path = os.path.join(folder_path, file_name) if file_name else None

        # Create a new binary file
        if operation == 'create':
            if not file_name:
                return False
            with open(file_path, 'wb') as file:
                pass  # Create an empty binary file
            return True

        # Write binary data to a file
        elif operation == 'write':
            if not file_name or data is None or not isinstance(data, bytes):
                return False
            with open(file_path, 'wb') as file:
                file.write(data)
            return True

        # Append binary data to a file
        elif operation == 'append':
            if not file_name or data is None or not isinstance(data, bytes):
                return False
            with open(file_path, 'ab') as file:
                file.write(data)
            return True

        # Read binary file content
        elif operation == 'read':
            if not file_name:
                return False
            with open(file_path, 'rb') as file:
                return file.read()

        # Clear binary file content
        elif operation == 'clear':
            if not file_name:
                return False
            with open(file_path, 'wb') as file:
                pass  # Overwrite with an empty binary file
            return True

        # Delete a binary file
        elif operation == 'delete':
            if not file_name:
                return False
            os.remove(file_path)
            return True

        # Rename a binary file
        elif operation == 'rename':
            if not file_name or not new_name:
                return False
            new_file_path = os.path.join(folder_path, new_name)
            os.rename(file_path, new_file_path)
            return True

        # List all files in the folder
        elif operation == 'list_files':
            return [f for f in os.listdir(folder_path) if os.path.isfile(os.path.join(folder_path, f))]

        else:
            return False  # Unsupported operation

    except (FileNotFoundError, PermissionError, OSError):
        return False




import ast
import os

def get_function_names_from_python_file(file_path):
    """Extract function names from a given Python file."""
    if not os.path.isfile(file_path):
        return False

    if not file_path.endswith(".py"):
        return False

    try:
        with open(file_path, "r") as file:
            file_content = file.read()
        
        # Parse the file content
        tree = ast.parse(file_content)
        
        # Extract function names
        function_names = [node.name for node in ast.walk(tree) if isinstance(node, ast.FunctionDef)]
        
        return ', '.join(function_names)  # Return function names as a comma-separated string
    except Exception as e:
        return False
